<?php

namespace TeamBooking\Mappers;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Database,
    TeamBooking\Toolkit,
    TeamBooking\Admin\Framework;

/**
 * Parses the reservation form data to a reservation object
 *
 * @param array $form_data
 *
 * @return \TeamBooking_ReservationData
 */
function reservationFormMapper($form_data)
{
    // Parse data
    $parsed_data = array();
    parse_str($form_data, $parsed_data);
    // verify nonce
    $nonce = $parsed_data['nonce'];
    if (!wp_verify_nonce($nonce, 'teambooking_submit_reservation')) {
        return FALSE;
    }
    try {
        $service = Functions\getSettings()->getService($parsed_data['service']);
    } catch (\Exception $e) {
        exit(Toolkit\wrapAjaxResponse($e->getMessage()));
    }
    $expected_fields = $service->getFormFields()->getHookList();
    $returned_fields = array();
    foreach ($expected_fields as $name) {
        // $name is the hook
        $form_field = new \TeamBooking_ReservationFormField();
        $form_field->setName($name);
        $form_field->setLabel($service->getFormFields()->getLabelFromHook($name));
        if (isset($parsed_data['form_fields'][ $name ])) {
            $form_field->setValue(Toolkit\filterInput($parsed_data['form_fields'][ $name ]));
            $form_field->setPriceIncrement($service->getFormFields()->getPriceIncrementFromOptionValue($name, $parsed_data['form_fields'][ $name ]));
        } else {
            // We are facing unchecked checkboxes here...
            $form_field->setValue(__('Not selected', 'team-booking'));
        }
        $returned_fields[] = $form_field;
    }
    $return = new \TeamBooking_ReservationData();
    $return->setServiceLocation($parsed_data['service_location']);
    $return->setTickets($parsed_data['tickets']);
    $return->setPostId($parsed_data['post_id']);
    $return->setPostTitle(get_the_title($parsed_data['post_id']));
    if (isset($parsed_data['owner'])) {
        $return->setCoworker($parsed_data['owner']);
    }
    $return->setPrice($service->getPrice());
    // Collecting discount campaigns
    $discounts_used = array();
    $discounted_price = Functions\getDiscountedPrice($service);
    $campaigns = Database\Promotions::getByClassAndService('campaign', $service->getId(), TRUE, TRUE);
    foreach ($campaigns as $pricing_id => $pricing) {
        $discounts_used[ $pricing_id ] = array(
            'name'  => $pricing->getName(),
            'value' => $pricing->getDiscount(),
            'type'  => $pricing->getDiscountType(),
        );
    }
    if (!empty($discounts_used)) {
        $return->setDiscount($discounts_used);
    }
    if ($service->getPrice() != $discounted_price) {
        if ($discounted_price < 0) {
            $discounted_price = 0;
        }
        $return->setPriceDiscounted($discounted_price);
    }
    $return->setServiceClass($service->getClass());
    $return->setCurrencyCode(Functions\getSettings()->getCurrencyCode());
    if (isset($parsed_data['event_id'])) {
        $return->setGoogleCalendarEvent($parsed_data['event_id']);
    }
    if (isset($parsed_data['calendar_id'])) {
        $return->setGoogleCalendarId(Toolkit\base64UrlDecode($parsed_data['calendar_id']));
    }
    $return->setIpnUniqueId(Toolkit\randomNumber(10) . current_time('timestamp', TRUE));
    $return->setServiceName($service->getName());
    $return->setServiceId($parsed_data['service']);
    $return->setFormFields($returned_fields);
    if (isset($parsed_data['event_id_parent'])) {
        if ($parsed_data['event_id_parent']) {
            $return->setGoogleCalendarEventParent($parsed_data['event_id_parent']);
        }
    }
    if (isset($parsed_data['slot_start'])) {
        $return->setSlotStart($parsed_data['slot_start']);
        $return->setStart(strtotime($parsed_data['slot_start']));
    }
    if (isset($parsed_data['slot_end'])) {
        $return->setSlotEnd($parsed_data['slot_end']);
        $return->setEnd(strtotime($parsed_data['slot_end']));
    }
    if (isset($parsed_data['customer_wp_id'])) {
        $return->setCustomerUserId($parsed_data['customer_wp_id']);
    }
    $now = Toolkit\getNowInSecondsUTC();
    $return->setCreationInstant($now);
    $return->setToken(Toolkit\generateToken());

    return $return;
}

/**
 * It takes a form field and returns the HTML for admin builder
 *
 * @param \TeamBookingFormTextField $field
 * @param                           $service_id
 *
 * @return string
 */
function adminFormCustomFieldsMapper(\TeamBookingFormTextField $field, $service_id)
{
    $fieldname = 'event[form_fields][custom]';
    ob_start();
    ?>
    <fieldset>
        <label for="<?= $fieldname ?>[<?= $field->getHook() ?>]">
            <input data-ays-ignore="true" name="<?= $fieldname ?>[<?= $field->getHook() ?>_show]" type="checkbox"
                   value="<?= $field->getHook() ?>" <?php checked(TRUE, $field->getIsActive()) ?>/><span> <?= esc_html__('Show', 'team-booking') ?></span><br>
            <?php if (get_class($field) !== 'TeamBookingFormRadio') { ?>
                <input data-ays-ignore="true" name="<?= $fieldname ?>[<?= $field->getHook() ?>_required]"
                       type="checkbox"
                       value="<?= $field->getHook() ?>" <?php checked(TRUE, $field->getIsRequired()) ?>/>
                <span> <?= esc_html__('Required', 'team-booking') ?></span><br>
            <?php } ?>
        </label>
    </fieldset>
    <?php
    $fieldset = ob_get_clean();
    ob_start();

    if (get_class($field) === 'TeamBookingFormTextField') {
        ?>
        <a title="<?= esc_attr__('Field validation rule', 'team-booking') ?>"
           id="tb-field-regex-open-<?= $field->getHook() ?>"
           class="ui mini circular label <?= $field->getValidationRegex() ? 'active' : 'inverted' ?>">
            <span class="dashicons dashicons-editor-spellcheck" style="vertical-align: middle;"></span>
        </a>
        <?= Functions\getValidationModal($field, $fieldname) ?>
    <?php } ?>
    <a title="<?= esc_attr__('Delete', 'team-booking') ?>" class="ui mini circular label tb-remove-custom-field"
       data-hook="<?= $field->getHook() ?>"
       data-serviceid="<?= $service_id ?>"
       href="#"><span class="dashicons dashicons-trash" style="vertical-align: middle;"></span></a>
    <a title="<?= esc_attr__('Save', 'team-booking') ?>" class="ui mini circular label tb-save-custom-field"
       data-hook="<?= $field->getHook() ?>" data-serviceid="<?= $service_id ?>"
       href="#"><span class="dashicons dashicons-yes" style="vertical-align: middle;"></span></a>
    <?php
    $buttons = ob_get_clean();
    ob_start();
    ?>
    <span class="dashicons dashicons-menu tb-drag-handle"></span>
    <span class="dashicons dashicons-arrow-down tb-expand-handle"></span>
    <span style="display: none;" class="dashicons dashicons-arrow-up tb-collapse-handle"></span>
    <?php
    $move_buttons = ob_get_clean();
    if (get_class($field) === 'TeamBookingFormTextField') {
        ob_start();
        ?>
        <table class="widefat tb-custom-field-draggable">
            <tbody>
            <tr>
                <td rowspan="3" class="tb-no-select" style="width: 20px;">
                    <?= $move_buttons ?>
                </td>
                <td rowspan="3" style="width:150px;">
                    <span
                        style="font-size: initial;"><strong><?= esc_html__('Text field', 'team-booking') ?></strong></span>
                    <!-- hidden content -->
                    <div class="tb-hide">
                        <br>
                        <?= $fieldset ?>
                    </div>
                </td>
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_label]" value="<?= $field->getLabel() ?>">
                    <span class="description"><?= esc_html__('Label', 'team-booking') ?></span>

                    <div style="float:right;">
                        <?= $buttons ?>
                    </div>
                </td>
            </tr>
            <!-- hidden content -->
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_default]" value="<?= $field->getValue() ?>"
                           placeholder="<?= esc_attr__('none', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Default value', 'team-booking') ?></span>
                </td>
            </tr>
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_hook]" value="<?= $field->getHook() ?>"
                           placeholder="<?= esc_attr__('without square brackets', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Hook', 'team-booking') ?></span>
                </td>
            </tr>
            </tbody>
        </table>
        <?php
        $return = ob_get_clean();
    } elseif (get_class($field) === 'TeamBookingFormTextarea') {
        /* @var $field \TeamBookingFormTextarea */
        ob_start();
        ?>
        <table class="widefat tb-custom-field-draggable">
            <tbody>
            <tr>
                <td rowspan="4" class="tb-no-select" style="width: 20px;">
                    <?= $move_buttons ?>
                </td>
                <td rowspan="4" style="width:150px;">
                    <span
                        style="font-size: initial;"><strong><?= esc_html__('Textarea', 'team-booking') ?></strong></span>
                    <!-- hidden content -->
                    <div class="tb-hide">
                        <br>
                        <?= $fieldset ?>
                    </div>
                </td>
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_label]" value="<?= $field->getLabel() ?>">
                    <span class="description"><?= esc_html__('Label', 'team-booking') ?></span>

                    <div style="float:right;">
                        <?= $buttons ?>
                    </div>
                </td>
            </tr>
            <!-- hidden content -->
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_default]" value="<?= $field->getValue() ?>"
                           placeholder="<?= esc_attr__('none', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Default text', 'team-booking') ?></span>
                </td>
            </tr>
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_hook]" value="<?= $field->getHook() ?>"
                           placeholder="<?= esc_attr__('without square brackets', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Hook', 'team-booking') ?></span>
                </td>
            </tr>
            </tbody>
        </table>
        <?php
        $return = ob_get_clean();
    } elseif (get_class($field) === 'TeamBookingFormCheckbox') {
        /* @var $field \TeamBookingFormCheckbox */
        ob_start();
        ?>
        <table class="widefat tb-custom-field-draggable">
            <tbody>
            <tr>
                <td rowspan="4" class="tb-no-select" style="width: 20px;">
                    <?= $move_buttons ?>
                </td>
                <td rowspan="4" style="width:150px;">
                    <span
                        style="font-size: initial;"><strong><?= esc_html__('Checkbox', 'team-booking') ?></strong></span>
                    <!-- hidden content -->
                    <div class="tb-hide">
                        <br>
                        <?= $fieldset ?>
                    </div>
                </td>
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_label]" value="<?= $field->getLabel() ?>">
                    <span class="description"><?= esc_html__('Label', 'team-booking') ?></span>

                    <div style="float:right;">
                        <?= $buttons ?>
                    </div>
                </td>
            </tr>
            <!-- hidden content -->
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_hook]" value="<?= $field->getHook() ?>"
                           placeholder="<?= esc_attr__('without square brackets', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Hook', 'team-booking') ?></span>
                </td>
            </tr>
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="checkbox" value="1"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_selected]" <?php checked(TRUE, $field->isChecked()) ?>>
                    <span class="description"><?= esc_html__('Selected by default', 'team-booking') ?></span>
                </td>
            </tr>
            </tbody>
        </table>
        <?php
        $return = ob_get_clean();
    } elseif (get_class($field) === 'TeamBookingFormSelect') {
        /* @var $field \TeamBookingFormSelect */
        ob_start();
        // legacy
        if (!($field->getValue() instanceof \TeamBooking_Components_Form_Option)) {
            $obj = new \TeamBooking_Components_Form_Option($field->getValue());
            $field->setValue($obj);
        }
        ?>
        <table class="widefat tb-custom-field-draggable">
            <tbody>
            <tr>
                <td rowspan="<?= 4 + count($field->getOptions()) ?>" class="tb-no-select" style="width: 20px;">
                    <?= $move_buttons ?>
                </td>
                <td rowspan="<?= 4 + count($field->getOptions()) ?>" style="width:150px;">
                    <span
                        style="font-size: initial;"><strong><?= esc_html__('Select', 'team-booking') ?></strong></span>
                    <!-- hidden content -->
                    <div class="tb-hide">
                        <br>
                        <?= $fieldset ?>
                    </div>
                </td>
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_label]" value="<?= $field->getLabel() ?>">
                    <span class="description"><?= esc_html__('Label', 'team-booking') ?></span>

                    <div style="float:right;">
                        <?= $buttons ?>
                    </div>
                </td>
            </tr>
            <!-- hidden content -->
            <tr class="tb-hide">
                <td>
                    <span
                        class="dashicons dashicons-admin-generic tb-advanced-option-handle <?= $field->getValue()->getPriceIncrement() > 0 ? 'active' : '' ?>"
                        data-modal="tbk-advanced-option-modal-<?= $field->getHook() ?>"></span>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_default][label]"
                           value="<?= $field->getValue()->getText() ?>"
                           placeholder="<?= esc_attr__('none', 'team-booking') ?>">
                    <input data-ays-ignore="true" type="hidden"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_default][price_increment]"
                           value="<?= $field->getValue()->getPriceIncrement() ?>">
                    <span class="description"><?= esc_html__('Option', 'team-booking') ?></span>
                </td>
            </tr>
            <?php
            foreach ($field->getOptions() as $key => $option) {
                // legacy
                if (!($option instanceof \TeamBooking_Components_Form_Option)) {
                    $option = new \TeamBooking_Components_Form_Option($option);
                }
                ?>
                <tr class="tb-hide">
                    <td>
                        <span class="dashicons dashicons-dismiss tb-delete-option-handle"></span>
                        <span
                            class="dashicons dashicons-admin-generic tb-advanced-option-handle <?= $option->getPriceIncrement() > 0 ? 'active' : '' ?>"
                            data-modal="tbk-advanced-option-modal-<?= $field->getHook() ?>"></span>

                        <input data-ays-ignore="true" type="text" class="all-options single-option"
                               name="<?= $fieldname ?>[<?= $field->getHook() ?>_options][<?= $key ?>][label]"
                               value="<?= $option->getText() ?>">
                        <input data-ays-ignore="true" type="hidden"
                               name="<?= $fieldname ?>[<?= $field->getHook() ?>_options][<?= $key ?>][price_increment]"
                               value="<?= $option->getPriceIncrement() ?>">
                        <span class="description"><?= esc_html__('Option', 'team-booking') ?></span>
                    </td>
                </tr>
            <?php } ?>
            <tr class="tb-hide">
                <td>
                    <span class="dashicons dashicons-plus-alt tb-add-option-handle"></span>
                    <span class="description">
                        <?= esc_html__('Add option', 'team-booking') ?>
                    </span>
                </td>
            </tr>
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_hook]" value="<?= $field->getHook() ?>"
                           placeholder="<?= esc_attr__('without square brackets', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Hook', 'team-booking') ?></span>
                </td>
            </tr>
            </tbody>
        </table>

        <!-- Option settings modal -->
        <?php ob_start() ?>
        <h4>
            <?= esc_html__('Price increment', 'team-booking') ?>
            <span class="description">
                (<?= esc_html__('base price', 'team-booking') ?>
                <?= Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), Functions\getSettings()->getService($service_id)->getPrice()) . ')' ?>
            </span>
        </h4>
        <input type="number" step="0.01" min="0" value="0" name="price_increment">
        <?= Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), NULL) ?>

        <?php
        $modal_content = ob_get_clean();
        $modal = new Framework\Modal('tbk-advanced-option-modal-' . $field->getHook());
        $modal->setButtonText(array('approve' => __('Set', 'team-booking') . ' (' . __('remember to save', 'team-booking') . ')', 'deny' => __('Close', 'team-booking')));
        $modal->setHeaderText(array('main' => $field->getLabel(), 'sub' => __('advanced option settings', 'team-booking')));
        $modal->addContent($modal_content);
        $modal->render();
        $return = ob_get_clean();

    } elseif (get_class($field) === 'TeamBookingFormRadio') {
        /* @var $field \TeamBookingFormRadio */
        ob_start();
        // legacy
        if (!($field->getValue() instanceof \TeamBooking_Components_Form_Option)) {
            $obj = new \TeamBooking_Components_Form_Option($field->getValue());
            $field->setValue($obj);
        }
        ?>
        <table class="widefat tb-custom-field-draggable">
            <tbody>
            <tr>
                <td rowspan="<?= 4 + count($field->getOptions()) ?>" class="tb-no-select" style="width: 20px;">
                    <?= $move_buttons ?>
                </td>
                <td rowspan="<?= 4 + count($field->getOptions()) ?>" style="width:150px;">
                    <span style="font-size: initial;"><strong>Radio group</strong></span>
                    <!-- hidden content -->
                    <div class="tb-hide">
                        <br>
                        <?= $fieldset ?>
                    </div>
                </td>
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_label]" value="<?= $field->getLabel() ?>">
                    <span class="description"><?= esc_html__('Label', 'team-booking') ?></span>

                    <div style="float:right;">
                        <?= $buttons ?>
                    </div>
                </td>
            </tr>
            <!-- hidden content -->
            <tr class="tb-hide">
                <td>
                    <span
                        class="dashicons dashicons-admin-generic tb-advanced-option-handle <?= $field->getValue()->getPriceIncrement() > 0 ? 'active' : '' ?>"
                        data-modal="tbk-advanced-option-modal-<?= $field->getHook() ?>"></span>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_default][label]"
                           value="<?= $field->getValue()->getText() ?>"
                           placeholder="<?= esc_attr__('none', 'team-booking') ?>">
                    <input data-ays-ignore="true" type="hidden"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_default][price_increment]"
                           value="<?= $field->getValue()->getPriceIncrement() ?>">
                    <span class="description"><?= esc_html__('Option', 'team-booking') ?></span>
                </td>
            </tr>
            <?php
            foreach ($field->getOptions() as $key => $option) {
                // legacy
                if (!($option instanceof \TeamBooking_Components_Form_Option)) {
                    $option = new \TeamBooking_Components_Form_Option($option);
                }
                ?>
                <tr class="tb-hide">
                    <td>

                        <span class="dashicons dashicons-dismiss tb-delete-option-handle"></span>
                        <span
                            class="dashicons dashicons-admin-generic tb-advanced-option-handle <?= $option->getPriceIncrement() > 0 ? 'active' : '' ?>"
                            data-modal="tbk-advanced-option-modal-<?= $field->getHook() ?>"></span>

                        <input data-ays-ignore="true" type="text" class="all-options single-option"
                               name="<?= $fieldname ?>[<?= $field->getHook() ?>_options][<?= $key ?>][label]"
                               value="<?= $option->getText() ?>">
                        <input data-ays-ignore="true" type="hidden"
                               name="<?= $fieldname ?>[<?= $field->getHook() ?>_options][<?= $key ?>][price_increment]"
                               value="<?= $option->getPriceIncrement() ?>">
                        <span class="description"><?= esc_html__('Option', 'team-booking') ?></span>
                    </td>
                </tr>
            <?php } ?>
            <tr class="tb-hide">
                <td>
                    <span class="dashicons dashicons-plus-alt tb-add-option-handle"></span>
                    <span class="description">
                        <?= esc_html__('Add option', 'team-booking') ?>
                    </span>
                </td>
            </tr>
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_hook]" value="<?= $field->getHook() ?>"
                           placeholder="<?= esc_attr__('without square brackets', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Hook', 'team-booking') ?></span>
                </td>
            </tr>
            </tbody>
        </table>

        <!-- Option settings modal -->
        <?php ob_start() ?>
        <h4>
            <?= esc_html__('Price increment', 'team-booking') ?>
            <span class="description">
                (<?= esc_html__('base price', 'team-booking') ?>
                <?= Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), Functions\getSettings()->getService($service_id)->getPrice()) . ')' ?>
            </span>
        </h4>
        <input type="number" step="0.01" min="0" value="0" name="price_increment">
        <?= Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), NULL) ?>

        <?php
        $modal_content = ob_get_clean();
        $modal = new Framework\Modal('tbk-advanced-option-modal-' . $field->getHook());
        $modal->setButtonText(array('approve' => __('Set', 'team-booking') . ' (' . __('remember to save', 'team-booking') . ')', 'deny' => __('Close', 'team-booking')));
        $modal->setHeaderText(array('main' => $field->getLabel(), 'sub' => __('advanced option settings', 'team-booking')));
        $modal->addContent($modal_content);
        $modal->render();
        $return = ob_get_clean();

    } elseif (get_class($field) === 'TeamBookingFormFileUpload') {
        /* @var $field \TeamBookingFormFileUpload */
        ob_start();
        ?>
        <table class="widefat tb-custom-field-draggable">
            <tbody>
            <tr>
                <td rowspan="3" class="tb-no-select" style="width: 20px;">
                    <?= $move_buttons ?>
                </td>
                <td rowspan="3" style="width:150px;">
                    <span
                        style="font-size: initial;"><strong><?= esc_html__('File Field', 'team-booking') ?></strong></span>
                    <!-- hidden content -->
                    <div class="tb-hide">
                        <br>
                        <?= $fieldset ?>
                    </div>
                </td>
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_label]" value="<?= $field->getLabel() ?>">
                    <span class="description"><?= esc_html__('Label', 'team-booking') ?></span>

                    <div style="float:right;">
                        <?= $buttons ?>
                    </div>
                </td>
            </tr>
            <!-- hidden content -->
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="<?= $fieldname ?>[<?= $field->getHook() ?>_hook]" value="<?= $field->getHook() ?>"
                           placeholder="<?= esc_attr__('without square brackets', 'team-booking') ?>">
                    <span class="description"><?= esc_html__('Hook', 'team-booking') ?></span>
                </td>
            </tr>
            <tr class="tb-hide">
                <td>
                    <input data-ays-ignore="true" type="text" class="all-options"
                           name="event[form_fields][custom][<?= $field->getHook() ?>_extensions]"
                           value="<?= $field->getFileExtensions(TRUE) ?>">
                    <span
                        class="description"><?= esc_html__('Allowed extensions (no dots, comma separated)', 'team-booking') ?></span>
                </td>
            </tr>
            </tbody>
        </table>
        <?php
        $return = ob_get_clean();
    }

    return $return;
}

/**
 * Used for abstractions
 *
 * @param \TeamBooking_ReservationData $data
 *
 * @return \TeamBooking\Slot
 * @throws \Exception
 */
function reservationDataToSlot(\TeamBooking_ReservationData $data)
{
    $slot = new \TeamBooking\Slot();
    $slot->setServiceId($data->getServiceId());
    $slot->setServiceName($data->getServiceName());
    $slot->setServiceInfo($data->getServiceDescription());
    $slot->setStartTime($data->getSlotStart());
    $slot->setEndTime($data->getSlotEnd());
    $slot->setEventId($data->getGoogleCalendarEvent());
    $slot->setCalendarId($data->getGoogleCalendarId());
    $slot->setEventIdParent($data->getGoogleCalendarEventParent());
    $slot->setAttendeesNumber($data->getTickets());
    $slot->setLocation($data->getServiceLocation());
    $slot->setAllDay(FALSE);
    $slot->setCoworkerId($data->getCoworker());
    if ($data->getServiceClass() === 'appointment' ||
        Functions\getSettings()->getService($data->getServiceId())->getAttendees() <= $slot->getAttendeesNumber()
    ) {
        $slot->setSoldout();
    }
    $slot->setFromReservation($data->getDatabaseId());

    return $slot;
}
