<?php

namespace TeamBooking\API;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Database,
    TeamBooking\Toolkit,
    TeamBooking\Files,
    TeamBooking\Calendar;

class REST
{
    private $params;
    private $response;
    /** @var  $service \TeamBookingType */
    private $service;

    private function parse($data)
    {
        if (isset($data['checksum'])) $this->params['checksum'] = filter_var($data['checksum'], FILTER_SANITIZE_STRING);
        if (isset($data['operation'])) $this->params['operation'] = filter_var($data['operation'], FILTER_SANITIZE_STRING);
        if (isset($data['gateway'])) $this->params['gateway'] = filter_var($data['gateway'], FILTER_SANITIZE_STRING);
        if (isset($data['id'])) $this->params['id'] = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);
        if (isset($data['coworker'])) $this->params['coworker'] = filter_var($data['coworker'], FILTER_SANITIZE_NUMBER_INT);
        if (isset($data['ctoken'])) $this->params['ctoken'] = filter_var($data['ctoken'], FILTER_SANITIZE_STRING);
        if (isset($data['auth_token'])) $this->params['auth_token'] = filter_var($data['auth_token'], FILTER_SANITIZE_STRING);
        if (isset($data['reservation'])) $this->params['reservation'] = filter_var($data['reservation']);
        if (isset($data['service_id'])) $this->params['service_id'] = filter_var($data['service_id'], FILTER_SANITIZE_STRING);
        if (isset($data['reason'])) $this->params['reason'] = filter_var($data['reason'], FILTER_SANITIZE_STRING);
        if (isset($data['service'])) $this->params['service'] = filter_var($data['service']);
        if (isset($data['show_confirm'])) {
            $this->params['show_confirm'] = filter_var($data['show_confirm'], FILTER_VALIDATE_BOOLEAN);
        } else {
            $this->params['show_confirm'] = FALSE;
        }
        if (isset($data['show_past'])) {
            $this->params['show_past'] = filter_var($data['show_past'], FILTER_VALIDATE_BOOLEAN);
        } else {
            $this->params['show_past'] = TRUE;
        }
    }

    public static function call($method, $data)
    {
        $api = new REST();
        $api->response['header'] = '403 Forbidden';
        $api->response['code'] = 403;
        $api->parse($data);
        if (!isset($api->params['operation'])) {
            $api->response['header'] = '400 Bad Request';
            $api->response['code'] = 400;
            $api->response['reason'] = 'missing parameters';
        } else {
            switch ($method) {
                case 'GET':
                    $api->GET();
                    break;
                case 'POST':
                    $api->POST();
                    break;
            }
        }

        return $api->response;
    }

    /**
     * @return bool|\TeamBooking_ReservationData
     */
    private function scopeReservation()
    {
        if (!$this->isRequestComplete(array('id', 'checksum'))) return FALSE;

        $reservation = Database\Reservations::getById($this->params['id']);
        if (!$reservation) {
            $this->response['header'] = '404 Not Found';
            $this->response['code'] = 404;

            return FALSE;
        }
        if ($reservation->getToken() !== $this->params['checksum']) {
            $this->response['reason'] = 'checksum fail';

            return FALSE;
        }

        try {
            $this->service = Functions\getSettings()->getService($reservation->getServiceId());
        } catch (\Exception $e) {
            $this->response['header'] = '404 Not Found';
            $this->response['code'] = 404;
            $this->response['reason'] = 'Service not found';

            return FALSE;
        }

        return $reservation;
    }

    private function GET()
    {
        switch ($this->params['operation']) {
            case 'get_reservation':
                if (!$this->isRequestComplete(array('auth_token', 'id'))) return FALSE;
                if (!$this->isTokenValid()) return FALSE;
                $reservation = Database\Reservations::getById($this->params['id']);
                if (!($reservation instanceof \TeamBooking_ReservationData)) {
                    $this->response['header'] = '404 Not Found';
                    $this->response['code'] = 404;
                    $this->response['reason'] = 'Record not found';

                    return FALSE;
                }
                Functions\getSettings()->incrementTokenUsage('get_reservation', $this->params['auth_token']);
                $this->response['header'] = '200 OK';
                $this->response['code'] = 200;
                $this->response['response'] = $reservation->getApiResource();
                break;
            case 'get_reservations':
                if (!$this->isRequestComplete(array('auth_token'))) return FALSE;
                if (!$this->isTokenValid()) return FALSE;
                if (isset($this->params['coworker'], $this->params['service_id'])) {
                    $reservations = Database\Reservations::getByServiceAndCoworker($this->params['service_id'], $this->params['coworker']);
                } else {
                    if (isset($this->params['coworker'])) {
                        $reservations = Database\Reservations::getByCoworker($this->params['coworker']);
                    } elseif (isset($this->params['service_id'])) {
                        $reservations = Database\Reservations::getByService($this->params['service_id']);
                    } else {
                        $reservations = Database\Reservations::getAll();
                    }
                }

                if (isset($this->params['show_past']) && !$this->params['show_past']) {
                    foreach ($reservations as $id => $reservation) {
                        if (Functions\isReservationPastInTime($reservation)) unset($reservations[ $id ]);
                    }
                }
                $results = array();
                foreach ($reservations as $id => $reservation) {
                    $results[] = $reservation->getApiResource();
                }
                Functions\getSettings()->incrementTokenUsage('get_reservations', $this->params['auth_token']);
                $this->response['header'] = '200 OK';
                $this->response['code'] = 200;
                $this->response['response'] = $results;
                break;
            case 'get_services':
                if (!$this->isRequestComplete(array('auth_token'))) return FALSE;
                if (!$this->isTokenValid()) return FALSE;
                $services = Functions\getSettings()->getServices();
                $results = array();
                foreach ($services as $service) {
                    $results[] = $service->getApiResource();
                }
                Functions\getSettings()->incrementTokenUsage('get_services', $this->params['auth_token']);
                $this->response['header'] = '200 OK';
                $this->response['code'] = 200;
                $this->response['response'] = $results;
                break;
            case 'get_slots':
                if (!$this->isRequestComplete(array('auth_token', 'service_id'))) return FALSE;
                if (!$this->isTokenValid()) return FALSE;
                try {
                    Functions\getSettings()->getService($this->params['service_id']);
                    Functions\getSettings()->incrementTokenUsage('get_slots', $this->params['auth_token']);
                    $calendar = new Calendar();
                    $slots = $calendar->getSlots(array($this->params['service_id']), Functions\getAuthCoworkersIdList())->getAllSlots();
                    $results = array();
                    foreach ($slots as $slot) {
                        $results[] = $slot->getApiResource();
                    }
                    $this->response['header'] = '200 OK';
                    $this->response['code'] = 200;
                    $this->response['response'] = $results;
                } catch (\Exception $e) {
                    $this->response['header'] = '404 Not Found';
                    $this->response['code'] = 404;
                    $this->response['reason'] = 'Service not found';
                }
                break;
            /**
             *  THIS IS RESERVED
             *  Customer payment e-mail link
             */
            case 'pay':
                $reservation = $this->scopeReservation();
                if (!$reservation) break;
                $this->showPayment($reservation);
                break;
            /**
             *  THIS IS RESERVED
             *  Customer get iCal file e-mail link
             */
            case 'ics':
                $reservation = $this->scopeReservation();
                if (!$reservation) break;
                if (Functions\getSettings()->getShowIcal()) {
                    Files\generateICSFile(
                        Toolkit\filterInput($reservation->getServiceName(), TRUE) . '.ics',
                        $reservation->getStart(),
                        $reservation->getEnd(),
                        $reservation->getServiceLocation(),
                        $this->service->getServiceInfo(),
                        $reservation->getUri(),
                        $reservation->getServiceName()
                    );
                    exit;
                }
                break;
            /**
             *  THIS IS RESERVED
             *  Customer cancellation e-mail link
             */
            case 'cancel':
                $reservation = $this->scopeReservation();
                if (!$reservation) break;
                if ($reservation->isCancelled()) {
                    $this->integrateWithTheme('<h1>' . esc_html__('This reservation is cancelled already!', 'team-booking') . '</h1>');
                    $this->response['output'] = TRUE;
                    break;
                }
                if ($this->service->getAllowCustomerCancellation()) {
                    $now = current_time('timestamp', TRUE);
                    if ($now + $this->service->getAllowCustomerCancellationUntil() < $reservation->getStart()) {
                        if ($this->params['show_confirm']) {
                            $ok_link = admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=cancel&id='
                                . $reservation->getDatabaseId()
                                . '&checksum=' . $reservation->getToken();
                            $this->showConfirmation(__('Do you want to cancel this reservation?', 'team-booking'), $ok_link, $reservation);
                        } else {
                            $this->cancel_reservation($reservation);
                            $this->integrateWithTheme('<h1>' . esc_html__('Reservation succesfully cancelled!', 'team-booking') . '</h1>');
                            $this->response['output'] = TRUE;
                        }
                    } else {
                        $this->integrateWithTheme('<h1>' . esc_html__('This reservation can no longer be cancelled!', 'team-booking') . '</h1>');
                        $this->response['output'] = TRUE;
                    }
                }
                break;
            /**
             *  THIS IS RESERVED
             *  Approval e-mail or gcal event link
             */
            case 'approve':
                $reservation = $this->scopeReservation();
                if (!$reservation) break;
                if (!Functions\getCoworkerFromApiToken($this->params['ctoken'])->isAdministrator()
                    && Functions\getSettings()->getCoworkerData($reservation->getCoworker())->getApiToken() !== $this->params['ctoken']
                ) break;

                if ($this->service->getApproveRule() === 'none') {
                    $this->integrateWithTheme('<h1>' . esc_html__('Approval not required!', 'team-booking') . '</h1>');
                    $this->response['output'] = TRUE;
                    break;
                }
                if ($this->params['show_confirm']) {
                    $ok_link = admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=approve&id='
                        . $reservation->getDatabaseId()
                        . '&ctoken=' . $this->params['ctoken']
                        . '&checksum=' . $reservation->getToken();
                    $this->showConfirmation(__('Do you want to approve this reservation?', 'team-booking'), $ok_link, $reservation);
                } else {
                    $this->approve_reservation($reservation);
                    $this->integrateWithTheme('<h1>' . esc_html__('Reservation succesfully approved!', 'team-booking') . '</h1>');
                    $this->response['output'] = TRUE;
                }
                break;
            /**
             *  THIS IS RESERVED
             *  Decline e-mail or gcal event link
             */
            case 'decline':
                $reservation = $this->scopeReservation();
                if (!$reservation) break;
                if (!Functions\getCoworkerFromApiToken($this->params['ctoken'])->isAdministrator()
                    && Functions\getSettings()->getCoworkerData($reservation->getCoworker())->getApiToken() !== $this->params['ctoken']
                ) break;
                if ($this->service->getApproveRule() === 'none') {
                    $this->integrateWithTheme('<h1>' . esc_html__('Approval not required!', 'team-booking') . '</h1>');
                    $this->response['output'] = TRUE;
                    break;
                }
                if ($this->params['show_confirm']) {
                    $ok_link = admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=decline&id='
                        . $reservation->getDatabaseId()
                        . '&ctoken=' . $this->params['ctoken']
                        . '&checksum=' . $reservation->getToken();
                    $this->showConfirmation(__('Do you want to decline this reservation?', 'team-booking'), $ok_link, $reservation);
                } else {
                    $who = Functions\getCoworkerFromApiToken($this->params['ctoken'])->getId();
                    $this->cancel_reservation($reservation, $who);
                    $this->integrateWithTheme('<h1>' . esc_html__('Reservation succesfully declined!', 'team-booking') . '</h1>');
                    $this->response['output'] = TRUE;
                }
                break;
            case 'test':
                break;
        }

        return TRUE;
    }

    private function POST()
    {
        switch ($this->params['operation']) {
            case 'edit_reservation':
                if (!$this->isRequestComplete(array('reservation', 'auth_token'))) return FALSE;
                if (!$this->isTokenValid(TRUE)) return FALSE;
                // getting rid of the slashes added by WordPress...
                $data = file_get_contents('php://input');
                parse_str($data, $data);
                if (get_magic_quotes_gpc()) {
                    $data = array_map('TeamBooking\Toolkit\stripslashes_deep', $data);
                }
                $this->params['reservation'] = $data['reservation'];
                // ... done!
                $resource_obj = json_decode($this->params['reservation']);
                if (!$this->validateReservationResource($resource_obj)) return FALSE;

                $reservation = Database\Reservations::getById($resource_obj->id);
                if (!$reservation) {
                    $this->response['header'] = '404 Not Found';
                    $this->response['code'] = 404;

                    return FALSE;
                }

                if (isset($resource_obj->customerUserID)) $reservation->setCustomerUserId($resource_obj->customerUserID);
                if (isset($resource_obj->customerTimezone)) $reservation->setCustomerTimezone($resource_obj->customerTimezone);
                if (isset($resource_obj->service->id)) {
                    try {
                        $this->service = Functions\getSettings()->getService($resource_obj->service->id);
                    } catch (\Exception $e) {
                        $this->response['header'] = '404 Not Found';
                        $this->response['code'] = 404;
                        $this->response['reason'] = 'Resource service not found';

                        return FALSE;
                    }
                    $reservation->setServiceId($resource_obj->service->id);
                    $reservation->setServiceName($this->service->getName());
                    $reservation->setServiceClass($this->service->getClass());
                    $reservation->setServiceDescription($this->service->getServiceInfo());
                }
                if (isset($resource_obj->service->location)) $reservation->setServiceLocation($resource_obj->service->location);
                if (isset($resource_obj->datetime->start)) $reservation->setStart($resource_obj->datetime->start);
                if (isset($resource_obj->datetime->end)) $reservation->setEnd($resource_obj->datetime->end);
                if (isset($resource_obj->payment->price)) $reservation->setPrice($resource_obj->payment->price);
                if (isset($resource_obj->payment->priceDiscounted)) $reservation->setPriceDiscounted($resource_obj->payment->priceDiscounted);
                if (isset($resource_obj->payment->isPaid)) $reservation->setPaid((bool)$resource_obj->payment->isPaid);
                if (isset($resource_obj->payment->currency)) $reservation->setCurrencyCode($resource_obj->payment->currency);
                if (isset($resource_obj->payment->gateway)) $reservation->setPaymentGateway($resource_obj->payment->gateway);
                if (isset($resource_obj->status)) {
                    if ($resource_obj->status === 'confirmed') $reservation->setStatusConfirmed();
                    if ($resource_obj->status === 'pending') $reservation->setStatusPending();
                    if ($resource_obj->status === 'cancelled') $reservation->setStatusCancelled();
                    if ($resource_obj->status === 'done') $reservation->setStatusDone();
                    if ($resource_obj->status === 'waiting_approval') $reservation->setStatusWaitingApproval();
                }
                if (isset($resource_obj->tickets)) $reservation->setTickets($resource_obj->tickets);
                if (isset($resource_obj->reminderSent)) $reservation->setEmailReminderSent((bool)$resource_obj->reminderSent);
                if (isset($resource_obj->referer->postID)) $reservation->setPostId($resource_obj->referer->postID);
                if (isset($resource_obj->referer->postID)) $reservation->setPostTitle($resource_obj->referer->postTitle);

                Database\Reservations::update($reservation);
                Functions\getSettings()->incrementTokenUsage('edit_reservation', $this->params['auth_token']);

                $this->response['header'] = '200 OK';
                $this->response['code'] = 200;

                break;
            case 'delete_reservation':
                if (!$this->isRequestComplete(array('id', 'auth_token'))) return FALSE;
                if (!$this->isTokenValid(TRUE)) return FALSE;
                Database\Reservations::delete($this->params['id']);
                Functions\getSettings()->incrementTokenUsage('delete_reservation', $this->params['auth_token']);
                $this->response['header'] = '200 OK';
                $this->response['code'] = 200;
                break;
            case 'cancel_reservation':
                if (!$this->isRequestComplete(array('id', 'auth_token'))) return FALSE;
                if (!$this->isTokenValid(TRUE)) return FALSE;
                $reservation = Database\Reservations::getById($this->params['id']);
                if (!$reservation) {
                    $this->response['header'] = '404 Not Found';
                    $this->response['code'] = 404;

                    return FALSE;
                }
                $this->cancel_reservation($reservation, $this->params['auth_token']);
                Functions\getSettings()->incrementTokenUsage('cancel_reservation', $this->params['auth_token']);
                break;
        }

        return TRUE;
    }

    private function cancel_reservation(\TeamBooking_ReservationData $reservation, $who = NULL)
    {
        if (isset($this->params['reason'])) $reservation->setCancellationReason($this->params['reason']);
        $process = new \TeamBooking_Reservation($reservation);
        $updated_record = $process->cancelReservation($this->params['id'], $who);
        if ($updated_record instanceof \TeamBooking_ReservationData) {
            // Everything went fine, let's update the database record
            Database\Reservations::update($updated_record);
            $this->response['header'] = '200 OK';
            $this->response['code'] = 200;
        } elseif ($updated_record instanceof \TeamBooking_Error) {
            // Something goes wrong
            if ($updated_record->getCode() == 7) {
                // The reservation is already cancelled, let's update the database record
                $reservation->setStatusCancelled();
                Database\Reservations::update($reservation);
            }
            $this->response['response'] = $updated_record->getMessage();
        }
    }

    private function approve_reservation(\TeamBooking_ReservationData $reservation)
    {
        $who = Functions\getCoworkerFromApiToken($this->params['ctoken'])->getId();
        $process = new \TeamBooking_Reservation($reservation);
        $updated_record = $process->doReservation($who);
        if ($updated_record instanceof \TeamBooking_ReservationData) {
            $this->response['header'] = '200 OK';
            $this->response['code'] = 200;
            $updated_record->setStatusConfirmed();
            Database\Reservations::update($updated_record);
            // Send e-mail messages
            if ($this->service->getApproveRule() === 'admin'
                && Functions\getSettings()->getCoworkerData($updated_record->getCoworker())->getCustomEventSettings($updated_record->getServiceId())->getGetDetailsByEmail()
            ) {
                $process->sendNotificationEmailToCoworker();
            }
            if ($this->service->getConfirm() && $updated_record->getCustomerEmail()) {
                $process->sendConfirmationEmail();
            }
        } elseif ($updated_record instanceof \TeamBooking_Error) {
            $this->response['response'] = $updated_record->getMessage();
        } else {
            $this->response['response'] = 'Not approvable (wrong service class)';
        }
    }

    private function showConfirmation($question_text, $ok_link, \TeamBooking_ReservationData $reservation)
    {
        $when_value = \DateTime::createFromFormat('U', $reservation->getStart());
        $when_value->setTimezone(new \DateTimeZone($reservation->getCustomerTimezone()));
        $service = Functions\getSettings()->getService($reservation->getServiceId());
        ob_start();
        ?>
        <h1><?= esc_html($question_text) ?></h1>
        <p><?= esc_html__('Reservation details', 'team-booking') ?></p>
        <table>
            <thead>
            <tr>
                <th>
                    <?= esc_html__('Service', 'team-booking') ?>
                </th>
                <th>
                    <?= esc_html__('When', 'team-booking') ?>
                </th>
            </tr>
            </thead>
            <tbody>
            <tr>
                <td>
                    <?= $reservation->getServiceName() ?>
                </td>
                <td>
                    <?= $when_value->format(get_option('date_format') . ' ' . get_option('time_format')) ?>
                    (<?= str_replace('_', ' ', $when_value->getTimezone()->getName()) ?>)
                </td>
            </tr>
            </tbody>
        </table>
        <?php if ($this->params['operation'] === 'cancel' && $service->getAllowCustomerCancellationReason()) { ?>
        <div>
            <p><?= esc_html__('If yes, please tell us why', 'team-booking') ?></p>
            <p><textarea class="cancellation-reason" style="width: auto"></textarea></p>
        </div>
        <script>
            jQuery(document).ready(function () {
                jQuery('.button.confirm').on('click keypress', function (e) {
                    if (e.which == 13 || e.which == 32 || e.which == 1) {
                        var reason = jQuery('.cancellation-reason').val();
                        if (reason.length !== 0) {
                            e.preventDefault();
                            jQuery(this).attr("href", jQuery(this).attr("href") + '&reason=' + reason);
                            window.location.href = jQuery(this).attr("href");
                        }
                    }
                })
            });
        </script>
    <?php } ?>
        <div class="buttons">
            <a class="button confirm" href="<?= $ok_link ?>">
                <?= esc_html__('yes', 'team-booking') ?>
            </a>
            <a class="button deny" href="<?= site_url() ?>">
                <?= esc_html__('no', 'team-booking') ?>
            </a>
        </div>
        <?php
        $this->integrateWithTheme(ob_get_clean());
        exit;
    }

    private function showPayment(\TeamBooking_ReservationData $reservation)
    {
        $gateways = Functions\getSettings()->getPaymentGatewaysActive();
        wp_enqueue_script('jquery');
        wp_enqueue_script('stripejs', 'https://js.stripe.com/v2/');
        ?>
        <html>
        <head>
            <link type="text/css" rel="stylesheet" href="<?= TEAMBOOKING_URL ?>/css/api-response.css">
            <link type="text/css" rel="stylesheet" href="<?= TEAMBOOKING_URL ?>/css/frontend.css">
            <?php wp_head(); ?>
        </head>
        <body style="visibility: visible;">
        <?= get_header() ?>
        <div class="tbk-api-wrapper">
            <?php
            if ($reservation->isPaid()) {
                echo '<h1>' . esc_html__('This reservation is paid already!') . '</h1>';
            } elseif ($reservation->isCancelled()) {
                echo '<h1>' . esc_html__('This reservation is cancelled.') . '</h1>';
            } elseif (empty($gateways)) {
                echo '<h1>' . esc_html__('There are currently no payment gateways active.') . '</h1>';
            } else {
                $when_value = \DateTime::createFromFormat('U', $reservation->getStart());
                $when_value->setTimezone(new \DateTimeZone($reservation->getCustomerTimezone()));
                ?>
                <h1><?= esc_html__('Please choose a payment method') ?></h1>
                <p><?= esc_html__('Reservation details', 'team-booking') ?></p>
                <div>
                    <table>
                        <thead>
                        <tr>
                            <th>
                                <?= esc_html__('Service', 'team-booking') ?>
                            </th>
                            <th>
                                <?= esc_html__('When', 'team-booking') ?>
                            </th>
                            <th>
                                <?= esc_html__('Amount', 'team-booking') ?>
                            </th>
                        </tr>
                        </thead>
                        <tbody>
                        <tr>
                            <td>
                                <?= $reservation->getServiceName() ?>
                            </td>
                            <td>
                                <?= $when_value->format(get_option('date_format') . ' ' . get_option('time_format')) ?>
                                (<?= str_replace('_', ' ', $when_value->getTimezone()->getName()) ?>)
                            </td>
                            <td>
                                <?= Functions\currencyCodeToSymbol($reservation->getCurrencyCode(), $reservation->getPriceIncremented()) ?>
                            </td>
                        </tr>
                        </tbody>
                    </table>
                </div>
                <div class="buttons">
                    <?php
                    foreach ($gateways as $gateway) {
                        echo $gateway->getPayButton();
                    }
                    ?>
                </div>
                <script>
                    jQuery(document).ready(function () {
                        jQuery('.wrapper').on('click keydown', '.tbk-pay-button', function (e) {
                            if (e.which == 13 || e.which == 32 || e.which == 1) {
                                var clicked_button = jQuery(this);
                                if (clicked_button.hasClass('tbk-loading')) {
                                    return;
                                }
                                clicked_button.addClass('tbk-loading');
                                var gateway_id = clicked_button.data('gateway');
                                var offsite = clicked_button.data('offsite');
                                var reservation_checksum = '<?= $reservation->getToken() ?>';
                                var reservation_database_id = <?= $reservation->getDatabaseId() ?>;
                                jQuery.post(
                                    '<?= admin_url('admin-ajax.php') ?>',
                                    {
                                        action                 : 'tb_submit_payment',
                                        reservation_checksum   : reservation_checksum,
                                        gateway_id             : gateway_id,
                                        reservation_database_id: reservation_database_id
                                    },
                                    function (response) {
                                        response = jQuery.trim(response.replace(/^\s*[\r\n]/gm, "").match(/!!TBK-START!!(.*[\s\S]*)!!TBK-END!!/)[1]);
                                        if (offsite == true || response.slice(0, 4) == 'http') {
                                            // redirecting
                                            window.location.href = response;
                                        } else {
                                            // parse the dinamically loaded scripts
                                            var content = jQuery(jQuery.parseHTML(response, document, true));
                                            // Load the response box
                                            clicked_button.closest('.buttons').html(content);
                                        }
                                    }
                                );
                            }
                        });
                    });
                </script>
                <?php
            }
            ?>
        </div>
        <?= get_footer() ?>
        </body>
        </html>
        <?php
        exit;
    }

    private function validateReservationResource($resource_obj)
    {
        $this->response['header'] = '400 Bad Request';
        $this->response['code'] = 400;
        $this->response['reason'] = 'invalid resource';

        if (!$resource_obj) return FALSE;
        if (isset($resource_obj->type)) {
            if ($resource_obj->type !== 'reservation') return FALSE;
        } else {
            return FALSE;
        }
        if (isset($resource_obj->customerUserID)) {
            if (filter_var($resource_obj->customerUserID, FILTER_VALIDATE_INT, array(
                    'options' => array('min_range' => 0)
                )) === FALSE
            ) {
                return FALSE;
            }
        }
        if (isset($resource_obj->customerTimezone) && !in_array($resource_obj->customerTimezone, \DateTimeZone::listIdentifiers())) {
            return FALSE;
        }
        if (isset($resource_obj->datetime->start)) {
            if (filter_var($resource_obj->datetime->start, FILTER_VALIDATE_INT, array(
                    'options' => array('min_range' => 0)
                )) === FALSE
            ) {
                return FALSE;
            }
        }
        if (isset($resource_obj->datetime->end)) {
            if (filter_var($resource_obj->datetime->end, FILTER_VALIDATE_INT, array(
                    'options' => array('min_range' => 0)
                )) === FALSE
            ) {
                return FALSE;
            }
        }
        if (isset($resource_obj->payment->price)) {
            if (filter_var($resource_obj->payment->price, FILTER_VALIDATE_INT, array(
                    'options' => array('min_range' => 0)
                )) === FALSE
            ) {
                return FALSE;
            }
        }
        if (isset($resource_obj->payment->priceDiscounted)) {
            if (filter_var($resource_obj->payment->priceDiscounted, FILTER_VALIDATE_INT, array(
                    'options' => array('min_range' => 0)
                )) === FALSE
            ) {
                return FALSE;
            }
        }
        if (isset($resource_obj->tickets)) {
            if (filter_var($resource_obj->tickets, FILTER_VALIDATE_INT, array(
                    'options' => array('min_range' => 0)
                )) === FALSE
            ) {
                return FALSE;
            }
        }
        if (isset($resource_obj->postID)) {
            if (filter_var($resource_obj->postID, FILTER_VALIDATE_INT, array(
                    'options' => array('min_range' => 0)
                )) === FALSE
            ) {
                return FALSE;
            }
        }

        // defaults
        $this->response['header'] = '403 Forbidden';
        $this->response['code'] = 403;
        unset($this->response['reason']);

        return TRUE;
    }

    private function isRequestComplete(array $params)
    {
        foreach ($params as $param) {
            if (!isset($this->params[ $param ])) {
                $this->response['header'] = '400 Bad Request';
                $this->response['code'] = 400;
                $this->response['reason'] = 'incomplete request';

                return FALSE;
            }
        }

        return TRUE;
    }

    private function isTokenValid($write = FALSE)
    {
        $tokens = Functions\getSettings()->getTokens();
        if (isset($tokens[ $this->params['auth_token'] ])) {
            if ($write && !$tokens[ $this->params['auth_token'] ]['write']) {
                $this->response['reason'] = 'read-only token';

                return FALSE;
            }

            return TRUE;
        } else {
            $this->response['reason'] = 'invalid token';

            return FALSE;
        }
    }

    /**
     * @param string $content
     */
    private function integrateWithTheme($content)
    {
        ?>
        <html>
        <head>
            <link type="text/css" rel="stylesheet" href="<?= TEAMBOOKING_URL ?>/css/api-response.css">
            <link type="text/css" rel="stylesheet" href="<?= TEAMBOOKING_URL ?>/css/frontend.css">
            <?php wp_head(); ?>
        </head>
        <body style="visibility: visible;">
        <?= get_header() ?>
        <?php Functions\api_page_main_content('<div class="tbk-api-wrapper">' . $content . '</div>') ?>
        <?= get_footer() ?>
        </body>
        </html>
        <?php
    }

}
