<?php

namespace TeamBooking;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Admin\Notices;

class Admin
{
    public function __construct()
    {
        // Admin panel callback registration
        add_action('admin_menu', array(
            $this,
            'addAdminMenu',
        ));
        // Admin panel form callback registration
        add_action('admin_post_option_submit', array(
            $this,
            'submitOptions',
        ));
        // Activate output buffer, needed until a better solution to wp_redirect "headers already sent" error will comes up
        add_action('init', array(
            $this,
            'activateOutputBuffer',
        ));
        add_action('admin_enqueue_scripts', array(
            $this,
            'addColorPicker',
        ));
        add_action('admin_enqueue_scripts', array(
            $this,
            'enqueueResources',
        ));
        // Add TinyMCE button
        add_action('admin_head', array(
            $this,
            'addTinyMCEButton',
        ));
        /*
         *  Clean reservation database
         */
        add_action('tb-db-cleaning-routine', 'TeamBooking\\Functions\\tbCleanReservationsRoutine');
    }

    public function addAdminMenu()
    {
        if (current_user_can('manage_options')) {
            // User is an admin coworker
            $new_reservation_ids = $this->getNewReservationIds();
            $bubble_title = __('new reservations', 'team-booking');
            $bubble_count = count($new_reservation_ids);
            $menu_label = 'TeamBooking '
                . "<span class='update-plugins tb-bubble count-$bubble_count' title='$bubble_title'><span class='update-count'>"
                . number_format_i18n($bubble_count)
                . '</span></span>';

            add_menu_page('Team Booking', $menu_label, 'manage_options', 'team-booking', array(
                $this,
                'createAdminPage',
            ), TEAMBOOKING_URL . 'images/admin-icon.png');

        } elseif (current_user_can('tb_can_sync_calendar')) {
            // User is a non-admin coworker
            $new_reservation_ids = $this->getNewReservationIds(FALSE);
            $bubble_title = __('new reservations', 'team-booking');
            $bubble_count = count($new_reservation_ids);
            $menu_label = 'TeamBooking '
                . "<span class='update-plugins tb-bubble count-$bubble_count' title='$bubble_title'><span class='update-count'>"
                . number_format_i18n($bubble_count)
                . '</span></span>';

            add_menu_page('Team Booking', $menu_label, 'tb_can_sync_calendar', 'team-booking', array(
                $this,
                'createCoworkerPage',
            ), TEAMBOOKING_URL . 'images/admin-icon.png');

        } else {
            // Anyone else
            return;
        }
    }

    /**
     * @param bool|TRUE $is_admin
     *
     * @return array
     */
    private function getNewReservationIds($is_admin = TRUE)
    {
        $transient = get_transient('teambooking_seen_reservations');
        if (!$transient) {
            $transient = array();
        }
        if (!isset($transient[ get_current_user_id() ])) {
            $transient[ get_current_user_id() ] = array();
        }

        if ($is_admin) {
            $reservation_ids = Database\Reservations::getIDs();
        } else {
            $reservation_ids = Database\Reservations::getIDsByCoworker(get_current_user_id());
        }

        return array_diff($reservation_ids, $transient[ get_current_user_id() ]);
    }

    /**
     * Fetch WP Roles
     *
     * @return array List of editable roles
     */
    private function fetchRoles()
    {
        return get_editable_roles();
    }

    /**
     * This method retrieve all the roles with tb_can_sync_calendar capability
     *
     * @return array List of roles allowed to link a calendar
     */
    public function getRolesWithSyncCap()
    {
        $roles = $this->fetchRoles();
        $list = array();
        foreach ($roles as $name => $role) {
            if (isset($role['capabilities']['tb_can_sync_calendar'])) {
                $list[ $name ] = $role['name'];
            }
        }

        return $list;
    }

    /**
     * Load WP Color Pickers
     */
    public function addColorPicker($hook)
    {
        // let's load only if we're on Team Booking dashboard
        if ($hook === 'toplevel_page_team-booking') {
            ///////////////////////
            // Color Picker CSS  //
            ///////////////////////   
            wp_enqueue_style('wp-color-picker');
        }
    }

    /**
     * Load admin resources
     */
    public function enqueueResources($hook)
    {
        // let's load only if we're on Team Booking dashboard
        if ($hook === 'toplevel_page_team-booking') {

            //////////////////////
            // jQuery resources //
            //////////////////////
            wp_enqueue_script('tb-admin-script', TEAMBOOKING_URL . 'js/admin.js', array(
                'jquery',
                'wp-color-picker',
                'jquery-ui-core',
                'jquery-ui-sortable',
            ), filemtime(TEAMBOOKING_PATH . 'js/admin.js'));
            // in javascript, object properties are accessed as ajax_object.some_value
            wp_localize_script('tb-admin-script', 'TB_vars', array(
                'post_url' => admin_url('admin-post.php'),
                'wpNonce'  => wp_create_nonce('team_booking_options_verify'),
            ));

            //////////////////////
            //   admin CSS      //
            //////////////////////
            wp_enqueue_style('tb-admin-style', TEAMBOOKING_URL . 'css/admin.css', array(), filemtime(TEAMBOOKING_PATH . 'css/admin.css'));

            ////////////////////////
            // semantic UI stuff  //
            ////////////////////////
            $comp_path = 'libs/semantic/components/';
            wp_enqueue_script('semantic-modal-script', TEAMBOOKING_URL . $comp_path . 'modal.js', array('jquery'));
            wp_enqueue_script('semantic-transition-script', TEAMBOOKING_URL . $comp_path . 'transition.js', array('jquery'));
            wp_enqueue_style('semantic-modal-style', TEAMBOOKING_URL . $comp_path . 'modal.css');
            wp_enqueue_style('semantic-transition-style', TEAMBOOKING_URL . $comp_path . 'transition.css');
            wp_enqueue_script('semantic-dimmer-script', TEAMBOOKING_URL . $comp_path . 'dimmer.js', array('jquery'));
            wp_enqueue_style('semantic-dimmer-style', TEAMBOOKING_URL . $comp_path . 'dimmer.css');
            wp_enqueue_style('semantic-button-style', TEAMBOOKING_URL . $comp_path . 'button.css');
            wp_enqueue_style('semantic-icon-style', TEAMBOOKING_URL . $comp_path . 'icon.css');
            wp_enqueue_script('semantic-checkbox-script', TEAMBOOKING_URL . $comp_path . 'checkbox.js', array('jquery'));
            wp_enqueue_style('semantic-checkbox-style', TEAMBOOKING_URL . $comp_path . 'checkbox.css');
            wp_enqueue_style('semantic-label-style', TEAMBOOKING_URL . $comp_path . 'label.css');

            /////////////////////////
            // jQuery AreYouSure?  //
            /////////////////////////
            wp_enqueue_script('tb-areyousure-js', TEAMBOOKING_URL . 'js/assets/jquery.are-you-sure.js', array('jquery'));

            /////////////////////////
            // jQuery autoNumeric  //
            /////////////////////////
            wp_enqueue_script('tb-autonumeric-js', TEAMBOOKING_URL . 'js/assets/autoNumeric.js', array('jquery'));

            /////////////////////////
            // jQuery tablesorter  //
            /////////////////////////
            wp_enqueue_script('tb-tablesorter-js', TEAMBOOKING_URL . 'js/assets/tablesorter/jquery.tablesorter.js', array('jquery'));
            wp_enqueue_script('tb-tablesorter-filter-js', TEAMBOOKING_URL . 'js/assets/tablesorter/widget-filter.js', array('jquery'));
            wp_enqueue_script('tb-tablesorter-pager-js', TEAMBOOKING_URL . 'js/assets/tablesorter/widget-pager.js', array('jquery'));
            wp_enqueue_script('tb-tablesorter-reflow-js', TEAMBOOKING_URL . 'js/assets/tablesorter/widget-reflow.js', array('jquery'));
            wp_enqueue_style('tb-tablesorter-style', TEAMBOOKING_URL . 'js/assets/tablesorter/css/theme.teambooking.css');

            ///////////////////////
            // jQuery flatpickr  //
            ///////////////////////
            wp_enqueue_script('tb-flatpickr-js', TEAMBOOKING_URL . 'libs/flatpickr/flatpickr.min.js', array('jquery'));
            wp_enqueue_style('tb-flatpickr-style', TEAMBOOKING_URL . 'libs/flatpickr/flatpickr.material_blue.min.css');
        }
        //////////////////////////////
        //   TinyMCE button CSS     //
        //////////////////////////////
        wp_enqueue_style('tb-tinymcebutton-style', TEAMBOOKING_URL . 'css/tinymce.css');
    }

    /**
     *  Activates output buffering
     */
    public function activateOutputBuffer()
    {
        ob_start();
    }

    /**
     * This method just intercept $_GET data looking for notices
     */
    public function checkForNotices()
    {
        require_once dirname(TEAMBOOKING_FILE_PATH) . '/includes/tb_admin_notices.php';

        if (isset($_GET['updated'])) {
            Notices\core_saved();
        }
        if (isset($_GET['settings_imported'])) {
            Notices\core_imported();
        }
        if (isset($_GET['version_mismatch'])) {
            Notices\core_version_mismatch();
        }
        if (isset($_GET['not_settings_file'])) {
            Notices\core_wrong_file();
        }
        if (isset($_GET['event_added'])) {
            Notices\service_added();
        }
        if (isset($_GET['event_updated'])) {
            Notices\service_updated();
        }
        if (isset($_GET['event_deleted'])) {
            Notices\service_deleted();
        }
        if (isset($_GET['selected_services_deleted'])) {
            Notices\services_deleted();
        }
        if (isset($_GET['custom_settings_updated'])) {
            Notices\personal_updated();
        }
        if (isset($_GET['reset'])) {
            Notices\personal_deleted();
        }
        if (isset($_GET['partialreset'])) {
            Notices\personal_partial();
        }
        if (isset($_GET['auth_success'])) {
            Notices\personal_success();
        }
        if (isset($_GET['auth_failed'])) {
            Notices\personal_failed();
        }
        if (isset($_GET['auth_no_refresh'])) {
            Notices\personal_no_refresh();
        }
        if (isset($_GET['auth_already_used'])) {
            Notices\personal_already_used();
        }
        if (isset($_GET['duplicated_linked_title'])) {
            Notices\personal_duplicated_event_title();
        }
        if (isset($_GET['reset_failed'])) {
            Notices\personal_revoke_failed();
        }
        if (isset($_GET['log_deleted'])) {
            Notices\overview_reservation_deleted();
        }
        if (isset($_GET['deleted_all'])) {
            Notices\overview_reservations_deleted();
        }
        if (isset($_GET['reservation_cancelled'])) {
            Notices\overview_reservation_cancelled();
        }
        if (isset($_GET['reservation_approved'])) {
            Notices\overview_reservation_confirmed();
        }
        if (isset($_GET['reservation_confirmed'])) {
            Notices\overview_reservation_confirmed();
        }
        if (isset($_GET['deleted_selected'])) {
            Notices\overview_reservations_deleted();
        }
        if (isset($_GET['not_revokable'])) {
            Notices\overview_reservation_not_revokable();
        }
        if (isset($_GET['generic_error'])) {
            Notices\overview_generic_error();
        }
        if (isset($_GET['error_logs_cleaned'])) {
            Notices\overview_logs_cleaned();
        }
        if (isset($_GET['token_added'])) {
            Notices\core_token_added();
        }
        if (isset($_GET['token_removed'])) {
            Notices\core_token_removed();
        }
        if (isset($_GET['database_repaired'])) {
            Notices\core_database_repaired();
        }

        $client_id = Functions\getSettings()->getApplicationClientId();
        $client_secret = Functions\getSettings()->getApplicationClientSecret();
        if (empty($client_id) || empty($client_secret)) {
            Notices\not_configured();

            return;
        }
        try {
            $timezone = new \DateTimeZone(get_option('timezone_string'));
        } catch (\Exception $ex) {
            Notices\timezone_approx();
        }
        // Sticky notice when PayPal Sandbox is active
        if (Functions\getSettings()->getPaymentGatewaySettingObject('paypal')->isActive()
            && Functions\getSettings()->getPaymentGatewaySettingObject('paypal')->isUseSandbox()
        ) {
            Notices\sandbox_active();
        }
    }

    /**
     * Builds the admin Teambooking page (Administrators)
     */
    public function createAdminPage()
    {
        ?>
        <div class="wrap">
            <h2 style="padding:0;"></h2><?php
            //Hack for Wordpress alerts that should stay always on top                                                                                                                                                                            
            // Checks for notices
            $this->checkForNotices();
            // Catch the what's new page
            if (isset($_GET['whatsnew'])) {
                Admin\Misc::render()->getWhatsnewPage();
            } else {
                // Defining tabs
                if (isset($_GET['tab'])) {
                    // Select the active tab, if present...
                    Admin\Misc::render()->getTabWrapper($_GET['tab']);
                    $function_to_call = 'buildAdminTab' . $_GET['tab'];
                    $this->$function_to_call();
                } else {
                    // ...or provide the default one
                    Admin\Misc::render()->getTabWrapper('overview');
                    $this->buildAdminTaboverview();
                }
            }
            ?>
        </div>
        <?php
    }

    /**
     * Builds the admin Teambooking page (Coworkers)
     */
    public function createCoworkerPage()
    {
        ?>
        <div class="wrap">
            <h2 style="padding:0;"></h2><?php
            //Hack for Wordpress alerts that should stay always on top                                                                                                                                                                              
            // Checks for notices
            $this->checkForNotices();
            // Defining allowed tabs
            $allowed_tabs = array(
                'overview',
                'events',
                'personal'
            );
            if (isset($_GET['tab'])) {
                // Whitelisting tab slug
                if (in_array($_GET['tab'], $allowed_tabs)) {
                    // Select the active tab, if present...
                    Admin\Misc::render()->getTabWrapperCoworker($_GET['tab']);
                    $function_to_call = 'buildAdminTab' . $_GET['tab'];
                    $this->$function_to_call();
                } else {
                    // No authorized/valid tab name, so collapse to default
                    Admin\Misc::render()->getTabWrapperCoworker('overview');
                    $this->buildAdminTaboverview();
                }
            } else {
                // ...or provide the default one
                Admin\Misc::render()->getTabWrapperCoworker('overview');
                $this->buildAdminTaboverview();
            }
            ?>
        </div>
        <?php
    }

    /**
     * Builds the general settings tab
     */
    public function buildAdminTabgeneral()
    {
        // Template call
        $content = new Admin\Core();
        // Set variables
        $content->roles_all = $this->fetchRoles();
        $content->roles_allowed = $this->getRolesWithSyncCap();
        // Render page
        echo $content->getPostBody();
    }

    /**
     * This method handles every admin form submission.
     *
     * Flow:
     * 1) Do authority check
     * 2) Do nonce check (field name: team_booking_options_verify)
     * 3) Do actions, choice based on submit button name (TODO: use $_POST['_wp_http_referer'] instead?)
     * 4) Redirect back to tab, along with the right footprint for admin notices
     * 5) Exit
     */
    public function submitOptions()
    {
        //////////////////////
        // Authority check  //
        //////////////////////
        if (!Functions\isAdminOrCoworker()
        ) {
            wp_die(__('You are not allowed to be on this page.', 'team-booking'));
        }

        //////////////////////
        //   Nonce check    //
        //////////////////////
        check_admin_referer('team_booking_options_verify');

        ////////////////////////
        // Admin only actions //
        ////////////////////////
        if (current_user_can('manage_options')) {
            if (isset($_POST['team_booking_options_submit'])) {
                $location = $this->saveCoreSettings();
            } elseif (isset($_POST['team_booking_insert_booking_type'])) {
                echo Toolkit\wrapAjaxResponse($this->addNewService());
                exit;
            } elseif (isset($_POST['team_booking_delete_booking_confirm'])) {
                echo Toolkit\wrapAjaxResponse($this->deleteService());
                exit;
            } elseif (isset($_POST['team_booking_confirm_pending_reservation_confirm'])) {
                echo Toolkit\wrapAjaxResponse($this->confirmPendingPaymentReservation());
                exit;
            } elseif (isset($_POST['team_booking_delete_all_reservations_confirm'])) {
                echo Toolkit\wrapAjaxResponse($this->deleteAllReservations());
                exit;
            } elseif (isset($_POST['team_booking_remove_selected_reservations'])) {
                echo Toolkit\wrapAjaxResponse($this->deleteSelectedReservations());
                exit;
            } elseif (isset($_POST['team_booking_remove_selected_services'])) {
                echo Toolkit\wrapAjaxResponse($this->deleteSelectedServices());
                exit;
            } elseif (isset($_POST['team_booking_save_coworker_settings'])) {
                $form_data = $_POST['form_data'];
                parse_str($form_data, $form_data_parsed);
                Functions\getSettings()->updateCoworkerUrl($_POST['coworker_id'], trim($form_data_parsed['coworker_url']));
                Functions\getSettings()->save();
                $location = admin_url('admin.php?page=team-booking&tab=coworkers&updated=1');
                echo Toolkit\wrapAjaxResponse($location);
                exit;
            } elseif (isset($_POST['team_booking_payment_options_submit'])) {
                $location = $this->savePaymentsSettings();
            } elseif (isset($_POST['set_tb_settings_backup'])) {
                $location = $this->importSettingsFromFile();
            } elseif (isset($_POST['team_booking_repair_database'])) {
                echo Toolkit\wrapAjaxResponse($this->repairDatabase());
                exit;
            } elseif (isset($_POST['set_google_core_from_json'])) {
                echo Toolkit\wrapAjaxResponse($this->importGoogleProjectFromJSON());
                exit;
            } elseif (isset($_POST['team_booking_create_api_token'])) {
                echo Toolkit\wrapAjaxResponse($this->createApiToken());
                exit;
            } elseif (isset($_POST['team_booking_revoke_api_token'])) {
                echo Toolkit\wrapAjaxResponse($this->revokeApiToken());
                exit;
            } elseif (isset($_POST['team_booking_add_custom_field'])) {
                echo Toolkit\wrapAjaxResponse($this->addServiceCustomField());
                exit;
            } elseif (isset($_POST['team_booking_remove_custom_field'])) {
                echo Toolkit\wrapAjaxResponse($this->removeServiceCustomField());
                exit;
            } elseif (isset($_POST['team_booking_save_custom_field'])) {
                echo Toolkit\wrapAjaxResponse($this->saveServiceCustomField());
                exit;
            } elseif (isset($_POST['team_booking_clone_service'])) {
                echo Toolkit\wrapAjaxResponse($this->cloneService());
                exit;
            } elseif (isset($_POST['tb_modify_reservation_save'])) {
                $reservation_database_id = $_POST['reservation_database_id'];
                $reservation = Database\Reservations::getById($reservation_database_id);

                if (!$_POST['reservation_payment_status']) {
                    $reservation->setPaid(FALSE);
                } else {
                    $reservation->setPaid(TRUE);
                }
                if (isset($_POST['reservation_unscheduled_status'])
                    && $reservation->getServiceClass() === 'service'
                ) {
                    if (!$_POST['reservation_unscheduled_status']) {
                        $reservation->setStatusConfirmed();
                    } else {
                        $reservation->setStatusDone();
                    }
                }
                Database\Reservations::update($reservation);
                $location = admin_url('admin.php?page=team-booking&tab=overview');
            } elseif (isset($_POST['team_booking_edit_reservation_data'])) {
                $reservation = Database\Reservations::getById($_POST['reservation_id']);
                $fields = $reservation->getFormFields();
                $new_fields = get_object_vars(json_decode(str_replace("\\", '', $_POST['fields'])));
                foreach ($fields as $key => $field) {
                    /** @var $field \TeamBooking_ReservationFormField */
                    $field->setValue(Toolkit\filterInput($new_fields[ $field->getName() ]));
                }
                $reservation->setFormFields($fields);
                Database\Reservations::update($reservation);
                echo Toolkit\wrapAjaxResponse('ok');
                exit;
            } elseif (isset($_POST['team_booking_insert_campaign'])) {
                echo Toolkit\wrapAjaxResponse($this->insertPromotionCampaign());
                exit;
            } elseif (isset($_POST['team_booking_insert_coupon'])) {
                echo Toolkit\wrapAjaxResponse($this->insertPromotionCoupon());
                exit;
            } elseif (isset($_POST['team_booking_edit_pricing'])) {
                echo Toolkit\wrapAjaxResponse($this->editPromotion());
                exit;
            } elseif (isset($_POST['team_booking_delete_pricing'])) {
                echo Toolkit\wrapAjaxResponse($this->deletePromotion());
                exit;
            } elseif (isset($_POST['team_booking_pricing_status_change'])) {
                echo Toolkit\wrapAjaxResponse($this->changePromotionStatus());
                exit;
            } elseif (isset($_POST['team_booking_reset_enumerable_reservations'])) {
                echo Toolkit\wrapAjaxResponse($this->resetEnumerableReservations());
                exit;
            } elseif (isset($_POST['team_booking_sync_all_google_calendars'])) {
                $calendars = Functions\getSettings()->getCoworkerData($_POST['coworker'])->getCalendars();
                foreach ($calendars as $calendar) {
                    Fetch\fromGoogle::fullSyncOf($calendar['calendar_id']);
                }
                $gcal_label = new Admin\Framework\TextLabel(__('ready', 'team-booking') . ' (' . count($calendars) . ')');
                $gcal_label->setColor('green');
                $gcal_label->setClass('tbk-calendars-state');
                echo Toolkit\wrapAjaxResponse($gcal_label->get());
                exit;
            } elseif (isset($_POST['get_tb_settings_backup'])) {
                echo Files\generateSettingsBackup();
                exit;
            } elseif (isset($_POST['team_booking_clone_service_suggestion'])) {
                echo Toolkit\wrapAjaxResponse($this->cloneServiceSuggestion());
                exit;
            } elseif (isset($_POST['team_booking_frontend_style_submit'])) {
                $location = $this->saveFrontendStyle();
            } elseif (isset($_POST['team_booking_move_custom_field'])) {
                echo Toolkit\wrapAjaxResponse($this->moveServiceCustomField());
                exit;
            } elseif (isset($_POST['team_booking_clean_error_logs'])) {
                echo Toolkit\wrapAjaxResponse($this->cleanErrorLogs());
                exit;
            } elseif (isset($_POST['team_booking_remove_coworker_residual'])) {
                echo Toolkit\wrapAjaxResponse($this->removeCoworkerResidualData());
                exit;
            } elseif (isset($_POST['team_booking_revoke_coworker_token_confirm'])) {
                echo Toolkit\wrapAjaxResponse($this->revokeCoworker($_POST['coworker_id']));
                exit;
            }
        }

        ////////////////////////////
        // Admin/Coworker actions //
        ////////////////////////////
        if (Functions\isAdminOrCoworker()) {
            if (isset($_POST['team_booking_approve_reservation_confirm'])) {
                echo Toolkit\wrapAjaxResponse($this->approveServiceReservation());
                exit;
            } elseif (isset($_POST['team_booking_delete_reservation_confirm'])) {
                if ($_POST['operation'] === 'delete') {
                    if (current_user_can('manage_options')) {
                        $location = $this->deleteServiceReservation();
                    }
                } else {
                    $location = $this->cancelServiceReservation();
                }
                echo Toolkit\wrapAjaxResponse($location);
                exit;
            } elseif (isset($_POST['team_booking_add_google_calendar'])) {
                echo Toolkit\wrapAjaxResponse($this->addGoogleCalendar());
                exit;
            } elseif (isset($_POST['team_booking_update_booking_type'])) {
                $location = $this->saveServiceSettings();
            } elseif (isset($_POST['team_booking_remove_google_calendar'])) {
                echo Toolkit\wrapAjaxResponse($this->removeGoogleCalendar());
                exit;
            } elseif (isset($_POST['team_booking_clean_google_calendar'])) {
                echo Toolkit\wrapAjaxResponse($this->cleanGoogleCalendar());
                exit;
            } elseif (isset($_POST['team_booking_sync_google_calendar'])) {
                Fetch\fromGoogle::fullSyncOf($_POST['calendar_id']);
                exit;
            } elseif (isset($_POST['team_booking_revoke_personal_token_confirm'])) {
                echo Toolkit\wrapAjaxResponse($this->revokeCoworker());
                exit;
            } elseif (isset($_POST['team_booking_save_personal_event_settings'])) {
                $location = $this->saveCoworkerServiceSettings();
            } elseif (isset($_POST['get_csv_file_tb'])) {
                echo Files\generateCSVFile($_POST['filter']);
                exit;
            } elseif (isset($_POST['get_customers_csv_file_tb'])) {
                echo Files\generateCSVClients();
                exit;
            } elseif (isset($_POST['get_xlsx_file_tb'])) {
                echo Files\generateXLSXFile($_POST['filter']);
                exit;
            } elseif (isset($_POST['get_customers_xlsx_file_tb'])) {
                echo Files\generateXLSXClients();
                exit;
            } elseif (isset($_POST['print-reservation-data_tb'])) {
                $reservation = Database\Reservations::getById($_POST['reservation_id']);
                echo Files\generateReservationPDF($reservation);
                exit;
            } elseif (isset($_POST['team_booking_toggle_service'])) {
                echo Toolkit\wrapAjaxResponse($this->toggleServiceActivation());
                exit;
            } elseif (isset($_POST['team_booking_toggle_calendar_independency'])) {
                echo Toolkit\wrapAjaxResponse($this->toggleCalendarIndependency());
                exit;
            } elseif (isset($_POST['team_booking_send_email_reminder_manually'])) {
                $send = Functions\sendEmailReminderManually($_POST['reservation_id']);
                echo Toolkit\wrapAjaxResponse($send ? 'ok' : 'ko');
                exit;
            }
        }

        ///////////////////////
        // Redirect and exit //
        ///////////////////////
        exit(wp_redirect($location));
    }

    public function buildAdminTabcoworkers()
    {
        $content = new Admin\Coworkers();
        echo $content->getPostBody();
    }

    public function buildAdminTabevents()
    {
        // Case: service details page
        if (isset($_GET['event'])) {
            // Checks if service_id exists, otherwise redirects to main list
            try {
                $service = Functions\getSettings()->getService($_GET['event']);
            } catch (\Exception $e) {
                die(wp_redirect(admin_url('admin.php?page=team-booking&tab=events')));
            }
            // Template call
            $content = new Admin\Service($service);
            // Render page
            if (isset($_GET['email'])) {
                echo $content->getPostBodyEmail();
            } elseif (isset($_GET['gcal'])) {
                echo $content->getPostBodyGcalSettings();
            } elseif (isset($_GET['form'])) {
                if (!current_user_can('manage_options')) {
                    die(wp_redirect(admin_url('admin.php?page=team-booking&tab=events')));
                }
                echo $content->getPostBodyReservationForm();
            } else {
                if (!current_user_can('manage_options')) {
                    die(wp_redirect(admin_url('admin.php?page=team-booking&tab=events')));
                }
                echo $content->getPostBody();
            }
        } // Case: services list
        else {
            // Template call
            $content = new Admin\Services();
            // Render content
            echo $content->getPostBody();
        }
    }

    public function buildAdminTabpersonal()
    {
        // Template call
        $content = new Admin\Personal();
        echo $content->getPostBody();
    }

    public function buildAdminTabaspect()
    {
        wp_enqueue_style('semantic-grid-style', TEAMBOOKING_URL . 'libs/semantic/components/grid.css');
        Loader::load_fonts();
        wp_enqueue_style('teambooking_fonts');
        // Template call
        $content = new Admin\Style();
        echo $content->getPostBody();
    }

    public function buildAdminTaboverview()
    {
        // Pending filter
        $filter_object = new Admin\FilterObject();
        if (!empty($_GET['filter'])) {
            $filter_object->decode($_GET['filter']);
        }

        // Syncing the events
        $process = new Fetch\fromGoogle();
        $process->sync();

        // Retrieving reservations
        if ($filter_object->getPendingReservations()) {
            $reservations = Database\Reservations::getAll(TRUE);

        } else {
            $reservations = Database\Reservations::getAll();

            // Update the transient for new reservations notification
            $transient = get_transient('teambooking_seen_reservations');
            if (!$transient) {
                $transient = array();
            }
            if (!isset($transient[ get_current_user_id() ])) {
                $transient[ get_current_user_id() ] = array();
            }
            $reservation_keys = array_keys($reservations);
            $new_reservation_ids = array_diff($reservation_keys, $transient[ get_current_user_id() ]);
            $transient[ get_current_user_id() ] = $reservation_keys;
            set_transient('teambooking_seen_reservations', $transient);
        }
        // Calling template
        $overview = new Admin\Overview($reservations);
        // Setting variables
        if (!isset($new_reservation_ids)) {
            $new_reservation_ids = array();
        }
        $overview->new_reservation_ids = $new_reservation_ids;
        $overview->filter_object = $filter_object;
        // Render
        echo $overview->getPostBody();
    }

    public function buildAdminTabpayments()
    {
        // Template call
        $content = new Admin\Payments();
        echo $content->getPostBody();
    }

    public function buildAdminTabpricing()
    {
        // Template call
        $content = new Admin\Promotion();
        echo $content->getPostBody();
    }

    public function buildAdminTabcustomers()
    {
        // Template call
        $content = new Admin\Customers();
        echo $content->getPostBody();
    }

    private function saveCoreSettings()
    {
        foreach ($_POST as $setting => $value) {
            if ($setting === 'client_id') Functions\getSettings()->setApplicationClientId(trim(filter_input(INPUT_POST, 'client_id', FILTER_SANITIZE_STRING)));
            if ($setting === 'client_secret') Functions\getSettings()->setApplicationClientSecret(trim(filter_input(INPUT_POST, 'client_secret', FILTER_SANITIZE_STRING)));
            if ($setting === 'project_name') Functions\getSettings()->setApplicationProjectName(filter_input(INPUT_POST, 'project_name', FILTER_SANITIZE_STRING));
            if ($setting === 'registration_url') Functions\getSettings()->setRegistrationUrl(filter_input(INPUT_POST, 'registration_url', FILTER_SANITIZE_URL));
            if ($setting === 'database_reservation_timeout') Functions\getSettings()->setDatabaseReservationTimeout(filter_input(INPUT_POST, 'database_reservation_timeout', FILTER_SANITIZE_NUMBER_INT));
            if ($setting === 'autofill') Functions\getSettings()->setAutofillReservationForm(filter_input(INPUT_POST, 'autofill', FILTER_SANITIZE_STRING));
            if ($setting === 'first_month_automatic') Functions\getSettings()->setShowFirstMonthWithFreeSlot(filter_input(INPUT_POST, 'first_month_automatic'));
            if ($setting === 'drop_tables') Functions\getSettings()->setDropTablesOnUninstall(filter_input(INPUT_POST, 'drop_tables'));
            if ($setting === 'show_ical') Functions\getSettings()->setShowIcal(filter_input(INPUT_POST, 'show_ical'));
            if ($setting === 'debug') Functions\getSettings()->setSilentDebug(filter_input(INPUT_POST, 'debug') === 'silent');
            if ($setting === 'gmaps_api_key') Functions\getSettings()->setGmapsApiKey(filter_input(INPUT_POST, 'gmaps_api_key', FILTER_SANITIZE_STRING));
            if ($setting === 'skip_gmaps') Functions\getSettings()->setSkipGmapLibs(filter_input(INPUT_POST, 'skip_gmaps'));
            if ($setting === 'max_pending_time') Functions\getSettings()->setMaxPendingTime(filter_input(INPUT_POST, 'max_pending_time', FILTER_VALIDATE_INT));
            if ($setting === 'roles_allowed') {
                $roles = $this->fetchRoles();
                global $wp_roles;
                // Clean capability assignment first...
                foreach ($roles as $name => $role) {
                    if ($name === 'administrator') continue;
                    $wp_roles->remove_cap($name, 'tb_can_sync_calendar');
                }
                //... and set new assignment after (if present)
                if (NULL !== filter_input(INPUT_POST, 'roles_allowed', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY)) {
                    foreach (filter_input(INPUT_POST, 'roles_allowed', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY) as $name => $role) {
                        $wp_roles->add_cap($name, 'tb_can_sync_calendar');
                    }
                }
            }
        }
        Functions\getSettings()->save();
        // Prepare redirect URL
        if (!isset($location)) {
            $location = admin_url('admin.php?page=team-booking&tab=general&updated=1');
        }

        return $location;
    }

    private function addNewService()
    {
        $response = '';
        if (empty($_POST['id'])
            || Functions\checkServiceIdExistance($_POST['id'])
            || Functions\checkServiceNameExistance(trim($_POST['name']))
        ) {
            if (empty($_POST['id'])
                || Functions\checkServiceIdExistance($_POST['id'])
            ) {
                $response .= 'id_fail '; // end space mandatory
            }
            if (Functions\checkServiceNameExistance($_POST['name'])) {
                $response .= 'name_fail';
            }
        } else {
            $service = new \TeamBookingType();
            $service->setName($_POST['name']);
            $service->setClass($_POST['class']);
            $service->setId($_POST['id']);
            $service->save();
            $response = 'ok';
        }

        return trim($response);
    }

    private function importSettingsFromFile()
    {
        $settings_object = unserialize(file_get_contents($_FILES['settings_backup_file']['tmp_name']));
        if ($settings_object instanceof \TeamBookingSettings) {
            $version_on_file = $settings_object->getVersion();
            if ($version_on_file <= TEAMBOOKING_VERSION) {
                // Replace settings
                update_option('team_booking', $settings_object);
                $location = admin_url('admin.php?page=team-booking&tab=general&settings_imported=1');
            } else {
                // Version mismatch
                $location = admin_url('admin.php?page=team-booking&tab=general&version_mismatch=1');
            }
        } else {
            // not the right file!
            $location = admin_url('admin.php?page=team-booking&tab=general&not_settings_file=1');
        }

        return $location;
    }

    private function repairDatabase()
    {
        Update::repairDatabase();

        return admin_url('admin.php?page=team-booking&tab=general&database_repaired=1');
    }

    private function importGoogleProjectFromJSON()
    {
        if (isset($_FILES['settings_json_file']['tmp_name'])) {
            $settings = json_decode(file_get_contents($_FILES['settings_json_file']['tmp_name']));
            if ($settings instanceof \stdClass) {
                if ($settings->web->client_id && $settings->web->client_secret && $settings->web->redirect_uris && $settings->web->javascript_origins) {
                    // URIs check
                    $redirect_uris = $settings->web->redirect_uris;
                    $uris_found = FALSE;
                    $js_origins = $settings->web->javascript_origins;
                    foreach ($redirect_uris as $redirect_uri) {
                        if ($redirect_uri == admin_url() . 'admin-ajax.php?action=teambooking_oauth_callback') {
                            $uris_found = TRUE;
                            break;
                        }
                    }
                    if ($uris_found) {
                        $uris_found = FALSE;
                        foreach ($js_origins as $js_origin) {
                            if ($js_origin == strtolower(substr($_SERVER['SERVER_PROTOCOL'], 0, strpos($_SERVER['SERVER_PROTOCOL'], '/'))) . '://' . $_SERVER['HTTP_HOST']) {
                                $uris_found = TRUE;
                                break;
                            }
                        }
                    }
                    if (!$uris_found) {
                        return 'uri mismatch';
                    }
                    Functions\getSettings()->setApplicationClientId($settings->web->client_id);
                    Functions\getSettings()->setApplicationClientSecret($settings->web->client_secret);
                    Functions\getSettings()->save();
                    $response = admin_url('admin.php?page=team-booking&tab=general&settings_imported=1');
                } else {
                    // Invalid file
                    $response = 'invalid file';
                }
            } else {
                // Invalid file
                $response = 'invalid file';
            }

            return $response;
        } else {
            return 'no file';
        }
    }

    private function saveServiceSettings()
    {
        $service = Functions\getSettings()->getService($_POST['service_id']);
        ////////////////////////////
        // General settings part  //
        ////////////////////////////
        if ($_POST['service_settings'] === 'general' && current_user_can('manage_options')) {
            foreach ($_POST['event'] as $setting => $value) {
                if ($setting === 'booking_type_name') $service->setName(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'max_attendees') $service->setAttendees(round(filter_var($value, FILTER_VALIDATE_INT)));
                if ($setting === 'max_tickets_per_reservation') $service->setMaxReservationsPerUser(min($service->getAttendees(), round(filter_var($value, FILTER_VALIDATE_INT))));
                if ($setting === 'show_reservations_left') $service->setShowReservationsLeft($value);
                if ($setting === 'service_color') $service->setServiceColor(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'info') $service->setServiceInfo($value); // late filtered
                if ($setting === 'direct_coworker') $service->setCoworkerForDirectAssignment(filter_var($value, FILTER_VALIDATE_INT));
                if ($setting === 'payment_must_be_done') $service->setPaymentMustBeDone(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'assignment_rule') $service->setAssignmentRule(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'allow_reservation') $service->setLoggedOnly($value);
                if ($setting === 'max_reservations_logged_user') $service->setMaxReservationsLoggedUser(filter_var($value, FILTER_VALIDATE_INT));
                if ($setting === 'show_times') {
                    $show_times = $service->getShowTimes();
                    $show_times->setValue($value);
                    $service->setShowTimes($show_times);
                }
                if ($setting === 'show_coworker') $service->setShowCoworker($value);
                if ($setting === 'show_coworker_url') $service->setShowCoworkerUrl($value);
                if ($setting === 'show_soldout') $service->setShowSoldout($value);
                if ($setting === 'service_price') $service->setPrice(round($value, 2));
                if ($setting === 'duration_rule') $service->setDurationRule(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'default_duration') $service->setDefaultDuration($value['hours'] * 3600 + $value['minutes'] * 60);
                if ($setting === 'location_setting') $service->setLocationSetting(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'location_address') $service->setLocationAddress(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'redirect') $service->setRedirect($value);
                if ($setting === 'redirect_url') $service->setRedirectUrl(trim(filter_var($value, FILTER_SANITIZE_URL)));
                if ($setting === 'approve_rule') $service->setApproveRule(filter_var($value, FILTER_SANITIZE_STRING));
                if ($setting === 'approve_until') $service->setFreeUntilApproval(filter_var($value, FILTER_VALIDATE_BOOLEAN));
                if ($setting === 'allow_customer_cancellation') $service->setAllowCustomerCancellation(filter_var($value, FILTER_VALIDATE_BOOLEAN));
                if ($setting === 'allow_customer_cancellation_reason') $service->setAllowCustomerCancellationReason(filter_var($value, FILTER_VALIDATE_BOOLEAN));
                if ($setting === 'allow_customer_cancellation_timespan') {
                    $service->setAllowCustomerCancellationUntil(
                        (int)$value['minutes'] * MINUTE_IN_SECONDS
                        + (int)$value['hours'] * HOUR_IN_SECONDS
                        + (int)$value['days'] * DAY_IN_SECONDS
                    );
                }
            }

            $location = admin_url('admin.php?page=team-booking&tab=events&event=' . $_POST['service_id'] . '&event_updated=1');
        }
        ////////////////////////////
        // Email settings part    //
        ////////////////////////////
        if ($_POST['service_settings'] === 'email') {
            if (current_user_can('manage_options')) {
                $service->setEmailForNotifications($_POST['event']['email_for_notifications']);
                isset($_POST['event']['email_to_email']) ? $service->setNotify(TRUE) : $service->setNotify(FALSE);
                isset($_POST['event']['email_to_customer']) ? $service->setConfirm(TRUE) : $service->setConfirm(FALSE);
                isset($_POST['event']['email_for_reminder']) ? $service->setSendReminderEmail(array('timeframe' => $_POST['event']['email_for_reminder_timeframe'])) : $service->setSendReminderEmail(array());
                $frontend_email = $service->getFrontendEmail();
                $frontend_email->setSubject($_POST['event']['front_end_email']['subject']);
                $frontend_email->setBody($_POST['event']['front_end_email']['body']);
                $service->setFrontendEmail($frontend_email);
                $backend_email = $service->getBackendEmail();
                $backend_email->setSubject($_POST['event']['back_end_email']['subject']);
                $backend_email->setBody($_POST['event']['back_end_email']['body']);
                $service->setBackendEmail($backend_email);
                $reminder_email = $service->getReminderEmail();
                $reminder_email->setSubject($_POST['event']['reminder_email']['subject']);
                $reminder_email->setBody($_POST['event']['reminder_email']['body']);
                $service->setReminderEmail($reminder_email);
                if ($_POST['event']['include_files_as_attachment'] == TRUE) {
                    $service->setIncludeFilesAsAttachment(TRUE);
                } else {
                    $service->setIncludeFilesAsAttachment(FALSE);
                }
                $service->setSendCancellationEmail($_POST['event']['send_cancellation_email']);
                $service->setSendCancellationEmailBackend($_POST['event']['send_cancellation_email_backend']);
                $cancellation_email = $service->getCancellationEmail();
                $cancellation_email->setSubject($_POST['event']['cancellation_email']['subject']);
                $cancellation_email->setBody($_POST['event']['cancellation_email']['body']);
                $service->setCancellationEmail($cancellation_email);
                $cancellation_email = $service->getCancellationEmailBackend();
                $cancellation_email->setSubject($_POST['event']['cancellation_email_backend']['subject']);
                $cancellation_email->setBody($_POST['event']['cancellation_email_backend']['body']);
                $service->setCancellationEmailBackend($cancellation_email);
            }
            $this->saveCoworkerServiceSettings(TRUE);
            $location = admin_url('admin.php?page=team-booking&tab=events&email=1&event=' . $_POST['service_id'] . '&event_updated=1');
        }
        ////////////////////////////
        // Reservation form part  //
        ////////////////////////////
        if ($_POST['service_settings'] === 'form' && current_user_can('manage_options')) {
            $form_fields = $service->getFormFields();
            // TODO: create a mapper for this
            isset($_POST['form_fields']['first_name']) ? $form_fields->first_name->setOn() : $form_fields->first_name->setOff();
            isset($_POST['form_fields']['first_name_required']) ? $form_fields->first_name->setRequiredOn() : $form_fields->first_name->setRequiredOff();
            if (isset($_POST['form_fields']['first_name_regex'])) {
                $form_fields->first_name->setValidationRegex(Toolkit\filterInput($_POST['form_fields']['first_name_regex']));
            }
            if (isset($_POST['form_fields']['first_name_validation'])) {
                $form_fields->first_name->setValidationRule($_POST['form_fields']['first_name_validation']);
            }
            isset($_POST['form_fields']['second_name']) ? $form_fields->second_name->setOn() : $form_fields->second_name->setOff();
            isset($_POST['form_fields']['second_name_required']) ? $form_fields->second_name->setRequiredOn() : $form_fields->second_name->setRequiredOff();
            if (isset($_POST['form_fields']['second_name_regex'])) {
                $form_fields->second_name->setValidationRegex(Toolkit\filterInput($_POST['form_fields']['second_name_regex']));
            }
            if (isset($_POST['form_fields']['second_name_validation'])) {
                $form_fields->second_name->setValidationRule($_POST['form_fields']['second_name_validation']);
            }
            isset($_POST['form_fields']['email']) ? $form_fields->email->setOn() : $form_fields->email->setOff();
            isset($_POST['form_fields']['email_required']) ? $form_fields->email->setRequiredOn() : $form_fields->email->setRequiredOff();
            if (isset($_POST['form_fields']['email_regex'])) {
                $form_fields->email->setValidationRegex(Toolkit\filterInput($_POST['form_fields']['email_regex']));
            }
            if (isset($_POST['form_fields']['email_validation'])) {
                $form_fields->email->setValidationRule($_POST['form_fields']['email_validation']);
            }
            isset($_POST['form_fields']['address']) ? $form_fields->address->setOn() : $form_fields->address->setOff();
            isset($_POST['form_fields']['address_required']) ? $form_fields->address->setRequiredOn() : $form_fields->address->setRequiredOff();
            if (isset($_POST['form_fields']['address_regex'])) {
                $form_fields->address->setValidationRegex(Toolkit\filterInput($_POST['form_fields']['address_regex']));
            }
            if (isset($_POST['form_fields']['address_validation'])) {
                $form_fields->address->setValidationRule($_POST['form_fields']['address_validation']);
            }
            isset($_POST['form_fields']['phone']) ? $form_fields->phone->setOn() : $form_fields->phone->setOff();
            isset($_POST['form_fields']['phone_required']) ? $form_fields->phone->setRequiredOn() : $form_fields->phone->setRequiredOff();
            if (isset($_POST['form_fields']['phone_regex'])) {
                $form_fields->phone->setValidationRegex(Toolkit\filterInput($_POST['form_fields']['phone_regex']));
            }
            if (isset($_POST['form_fields']['phone_validation'])) {
                $form_fields->phone->setValidationRule($_POST['form_fields']['phone_validation']);
            }
            isset($_POST['form_fields']['url']) ? $form_fields->url->setOn() : $form_fields->url->setOff();
            isset($_POST['form_fields']['url_required']) ? $form_fields->url->setRequiredOn() : $form_fields->url->setRequiredOff();
            if (isset($_POST['form_fields']['url_regex'])) {
                $form_fields->url->setValidationRegex(Toolkit\filterInput($_POST['form_fields']['url_regex']));
            }
            if (isset($_POST['form_fields']['url_validation'])) {
                $form_fields->url->setValidationRule($_POST['form_fields']['url_validation']);
            }
            $service->setFormFields($form_fields);
            $location = admin_url('admin.php?page=team-booking&tab=events&form=1&event=' . $_POST['service_id'] . '&event_updated=1');
        }
        $service->save();

        return $location;
    }

    private function savePaymentsSettings()
    {
        if (isset($_POST['currency_code']) && !empty($_POST['currency_code'])) {
            Functions\getSettings()->setCurrencyCode($_POST['currency_code']);
        }
        if (isset($_POST['gateway_settings'])) {
            foreach ($_POST['gateway_settings'] as $gateway_id => $settings) {
                $object = Functions\getSettings()->getPaymentGatewaySettingObject($gateway_id);
                if ($object instanceof \TeamBooking_PaymentGateways_Settings) {
                    $object->saveBackendSettings($settings, Functions\getSettings()->getCurrencyCode());
                }
            }
        }
        Functions\getSettings()->save();

        return admin_url('admin.php?page=team-booking&tab=payments&updated=1');
    }

    private function deleteService()
    {
        Functions\getSettings()->dropService($_POST['booking_id']);
        $coworkers = Functions\getCoworkersIdList();
        foreach ($coworkers as $coworker) {
            Functions\getSettings()->getCoworkerData($coworker)->dropCustomServiceSettings($_POST['booking_id']);
        }
        Functions\getSettings()->save();

        return admin_url('admin.php?page=team-booking&tab=events&event_deleted=1');
    }

    private function cleanErrorLogs()
    {
        $error_logs = array();
        Functions\getSettings()->setErrorLogs($error_logs);
        Functions\getSettings()->save();
        $location = admin_url('admin.php?page=team-booking&tab=overview&error_logs_cleaned=1');

        return $location;
    }

    private function deleteServiceReservation()
    {
        $filter = new Admin\FilterObject();
        if (isset($_POST['is_pending'])) {
            $filter->setPendingReservations(TRUE);
        }
        // TODO: cancel
        Database\Reservations::delete($_POST['reservation_id']);
        $url = 'admin.php?page=team-booking&tab=overview&log_deleted=1&filter=' . $filter->encode();

        return admin_url($url);
    }

    private function approveServiceReservation()
    {
        /**
         * Retrieving the reservation record in database
         */
        $reservation_db_record = Database\Reservations::getById($_POST['reservation_id']);

        try {
            $service = Functions\getSettings()->getService($reservation_db_record->getServiceId());
        } catch (\Exception $e) {
            return admin_url('admin.php?page=team-booking&tab=overview&generic_error=' . urlencode($e->getMessage()));
        }

        if (current_user_can('manage_options')
            ||
            ($service->getApproveRule() === 'coworker'
                && get_current_user_id() == $reservation_db_record->getCoworker()
            )
        ) {

            /**
             * Instantiating the reservation service class
             */
            $reservation = new \TeamBooking_Reservation($reservation_db_record);
            /**
             * Do the reservation
             */
            $updated_record = $reservation->doReservation();
            if ($updated_record instanceof \TeamBooking_ReservationData) {
                /**
                 * Everything went fine, let's update the database record
                 */
                $updated_record->setStatusConfirmed();
                Database\Reservations::update($updated_record);

                // Send e-mail messages
                if ($service->getApproveRule() === 'admin'
                    && Functions\getSettings()->getCoworkerData($updated_record->getCoworker())->getCustomEventSettings($updated_record->getServiceId())->getGetDetailsByEmail()
                ) {
                    $reservation->sendNotificationEmailToCoworker();
                }
                if ($updated_record->getCustomerEmail() && $reservation->getServiceObj()->getConfirm()) {
                    $reservation->sendConfirmationEmail();
                }

                $location = admin_url('admin.php?page=team-booking&tab=overview&reservation_approved=1');
            } elseif ($updated_record instanceof \TeamBooking_Error) {
                /**
                 * Something goes wrong
                 */
                $message = $updated_record->getMessage();
                $location = 'ERROR: ' . $message;
            } else {
                /**
                 * Not an Appointment, nor Event
                 */
                $location = admin_url('admin.php?page=team-booking&tab=overview&not_approvable=1');
            }

            return $location;
        } else {
            return admin_url('admin.php?page=team-booking&tab=overview&generic_error=unauthorized');
        }
    }

    private function confirmPendingPaymentReservation()
    {
        /**
         * Retrieving the reservation record
         */
        $reservation_db_record = Database\Reservations::getById($_POST['reservation_id']);
        /**
         * Instantiating the reservation service class
         */
        $reservation = new \TeamBooking_Reservation($reservation_db_record);
        /**
         * Do the reservation
         */
        $updated_record = $reservation->doReservation();
        if ($updated_record instanceof \TeamBooking_ReservationData) {
            /**
             * Everything went fine, let's update the database record
             */
            $updated_record->setStatusConfirmed();
            if ($_POST['set_as_paid'] === 'true') {
                $updated_record->setPaid(TRUE);
            } else {
                $updated_record->setPaid(FALSE);
            }
            Database\Reservations::update($updated_record);

            // Send e-mail messages
            if ($reservation->getServiceObj()->getNotify()) {
                $reservation->sendNotificationEmail();
            }
            if (Functions\getSettings()->getCoworkerData($updated_record->getCoworker())->getCustomEventSettings($updated_record->getServiceId())->getGetDetailsByEmail()) {
                $reservation->sendNotificationEmailToCoworker();
            }
            if ($updated_record->getCustomerEmail() && $reservation->getServiceObj()->getConfirm()) {
                $reservation->sendConfirmationEmail();
            }

            $location = admin_url('admin.php?page=team-booking&tab=overview&reservation_confirmed=1');
        } elseif ($updated_record instanceof \TeamBooking_Error) {
            /**
             * Something goes wrong
             */
            $message = $updated_record->getMessage();
            $location = 'ERROR: ' . $message;
        } else {
            /**
             * Not an Appointment, nor Event
             */
            $location = admin_url('admin.php?page=team-booking&tab=overview&not_approvable=1');
        }

        return $location;
    }

    private function cancelServiceReservation()
    {
        /**
         * Retrieving the reservation record in database
         */
        $reservation_db_record = Database\Reservations::getById($_POST['reservation_id']);
        if (!$reservation_db_record) return admin_url('admin.php?page=team-booking&tab=overview&filter=' . $_POST['filter'] . '&generic_error=1');
        if (!current_user_can('manage_options')) {
            try {
                if ($reservation_db_record->getCoworker() != get_current_user_id()
                    || Functions\getSettings()->getService($reservation_db_record->getServiceId())->getApproveRule() !== 'coworker'
                ) {
                    return admin_url('admin.php?page=team-booking&tab=overview&filter=' . $_POST['filter'] . '&not_revokable=1');
                }
            } catch (\Exception $e) {
                return admin_url('admin.php?page=team-booking&tab=overview&filter=' . $_POST['filter'] . '&not_revokable=1');
            }
        }
        /**
         * Instantiating the reservation service class
         */
        $reservation = new \TeamBooking_Reservation($reservation_db_record);
        /**
         * Calling the cancel method
         */
        $updated_record = $reservation->cancelReservation($_POST['reservation_id'], get_current_user_id());
        if ($updated_record instanceof \TeamBooking_ReservationData) {
            /**
             * Everything went fine, let's update the database record
             */
            Database\Reservations::update($updated_record);
            $location = admin_url('admin.php?page=team-booking&tab=overview&filter=' . $_POST['filter'] . '&reservation_cancelled=1');
        } elseif ($updated_record instanceof \TeamBooking_Error) {
            /**
             * Something goes wrong
             */
            if ($updated_record->getCode() == 7) {
                /*
                 * The reservation is already cancelled, let's update the database record
                 */
                $reservation_db_record->setStatusCancelled();
                Database\Reservations::update($reservation_db_record);
            }
            $message = urlencode($updated_record->getMessage());
            $location = admin_url('admin.php?page=team-booking&tab=overview&filter=' . $_POST['filter'] . '&generic_error=' . $message);
        } else {
            /**
             * Not an Appointment, nor Event
             */
            $location = admin_url('admin.php?page=team-booking&tab=overview&filter=' . $_POST['filter'] . '&not_revokable=1');
        }

        return $location;
    }

    private function deleteSelectedServices()
    {
        $services = json_decode(str_replace("\\", '', $_POST['services']));
        $coworkers = Functions\getCoworkersIdList();
        foreach ($services as $service_id) {
            Functions\getSettings()->dropService($service_id);
            foreach ($coworkers as $coworker) {
                Functions\getSettings()->getCoworkerData($coworker)->dropCustomServiceSettings($service_id);
            }
        }
        Functions\getSettings()->save();
        $location = admin_url('admin.php?page=team-booking&tab=events&selected_services_deleted=1');

        return $location;

    }

    private function deleteSelectedReservations()
    {
        $reservations = json_decode($_POST['reservations']);
        if (isset($_POST['is_pending'])) {
            $filter = new Admin\FilterObject();
            $filter->setPendingReservations(TRUE);
            $url = 'admin.php?page=team-booking&tab=overview&deleted_selected=1&filter=' . $filter->encode();
        } else {
            $url = 'admin.php?page=team-booking&tab=overview&deleted_selected=1';
        }
        Database\Reservations::delete($reservations);

        return admin_url($url);
    }

    private function deleteAllReservations()
    {
        if ($_POST['is_pending']) {
            Functions\cleanReservations(TRUE, FALSE, FALSE);
            $filter = new Admin\FilterObject();
            $filter->setPendingReservations(TRUE);
            $url = 'admin.php?page=team-booking&tab=overview&deleted_all=1&filter=' . $filter->encode();
        } else {
            Functions\cleanReservations(TRUE);
            $url = 'admin.php?page=team-booking&tab=overview&deleted_all=1';
        }

        return admin_url($url);
    }

    private function addGoogleCalendar()
    {
        $coworker = Functions\getSettings()->getCoworkerData(get_current_user_id());
        /**
         * Let's test if the Google Calendar ID belongs to
         * the authorized Google Account
         *
         * Even if the IDs are already picked in the right Google Account,
         * this second step provides a better security level.
         */
        $calendar = new Calendar();
        $test = $calendar->testCalendarID(get_current_user_id(), trim($_POST['calendar_id']));
        if ($test === FALSE) {
            return 404;
        } else {
            $coworker->addCalendarId(trim($_POST['calendar_id']));
            Functions\getSettings()->save();
            Fetch\fromGoogle::fullSyncOf($_POST['calendar_id']);
            $location = admin_url('admin.php?page=team-booking&tab=personal&custom_settings_updated=1');

            return $location;
        }
    }

    private function removeGoogleCalendar()
    {
        $coworker = Functions\getSettings()->getCoworkerData(get_current_user_id());
        if ($coworker->dropCalendarId(trim($_POST['calendar_id_key']))) {
            Database\Events::removeCalendar(trim($_POST['calendar_id_key']));
        }
        Functions\getSettings()->save();
        $location = admin_url('admin.php?page=team-booking&tab=personal&custom_settings_updated=1');

        return $location;
    }

    private function cleanGoogleCalendar()
    {
        \TeamBooking\Fetch\fromGoogle::deletePastEventsOf(trim($_POST['calendar_id_key']));
        $location = admin_url('admin.php?page=team-booking&tab=personal&custom_settings_updated=1');

        return $location;
    }

    private function removeCoworkerResidualData()
    {
        Database\Events::removeCoworker($_POST['coworker_id']);
        Functions\getSettings()->dropCoworkerData($_POST['coworker_id']);
        Functions\getSettings()->save();

        return admin_url('admin.php?page=team-booking&tab=coworkers&updated=1');
    }

    private function revokeCoworker($coworker_id = FALSE)
    {
        if (!$coworker_id) {
            $coworker = Functions\getSettings()->getCoworkerData(get_current_user_id());
            $location_root = 'admin.php?page=team-booking&tab=personal';
        } else {
            $coworker = Functions\getSettings()->getCoworkerData($coworker_id);
            $location_root = 'admin.php?page=team-booking&tab=coworkers';
        }
        $location = admin_url($location_root . '&reset=1');
        // Prepare for programmatically revoke token
        if (isset(json_decode($coworker->getAccessToken())->refresh_token)) {
            $token = json_decode($coworker->getAccessToken())->refresh_token;
            // Revoke token
            $calendar_class = new Calendar();
            $revoke_attempt = $calendar_class->revokeToken($token);
            if ($revoke_attempt !== TRUE) {
                $location = admin_url($location_root . '&reset_failed=' . urlencode($revoke_attempt));

                return $location;
            }
        } else {
            $location = admin_url($location_root . '&partialreset=1');
        }
        // Erase Access Token
        $coworker->setAccessToken('');
        // Remove Google Calendar ID(s)
        $coworker->dropAllCalendarIds();
        // Remove the events from database
        Database\Events::removeCoworker($coworker_id);
        Functions\getSettings()->save();

        return $location;
    }

    private function saveCoworkerServiceSettings($email = FALSE)
    {
        $coworker_data = Functions\getSettings()->getCoworkerData(get_current_user_id());
        $settings = $coworker_data->getCustomEventSettings($_POST['service_id']);
        /////////////////
        // Email part  //
        /////////////////
        if ($email) {
            $settings->setGetDetailsByEmail((int)$_POST['data']['get_details_by_email']);
            $settings->setIncludeFilesAsAttachment((int)$_POST['data']['include_uploaded_files_as_attachment']);
            $settings->setNotificationEmailBody($_POST['data']['email']['email_text']['body']);
            $settings->setNotificationEmailSubject($_POST['data']['email']['email_text']['subject']);
            $location = admin_url('admin.php?page=team-booking&tab=events&email=1&event=' . $_POST['service_id'] . '&event_updated=1');
        }
        ///////////////////////////
        // Google Calendar part  //
        ///////////////////////////
        if (!$email) {
            $services_id_list = Functions\getSettings()->getServiceIdList();
            $other_linked_event_titles = array();
            foreach ($services_id_list as $id) {
                if ($id != $_POST['service_id']) {
                    $other_linked_event_titles[] = strtolower($coworker_data->getCustomEventSettings($id)->getLinkedEventTitle());
                }
            }
            $linked_event_title_lowercase = strtolower($_POST['data']['linked_event_title']);

            // Checks for duplicate linked event title
            if (in_array($linked_event_title_lowercase, $other_linked_event_titles)
                && !Functions\getSettings()->getService($_POST['service_id'])->isClass('service')
            ) {
                $location = admin_url('admin.php?page=team-booking&tab=events&gcal=1&event=' . $_POST['service_id'] . '&duplicated_linked_title=1');
            } else {
                if (isset($_POST['data']['booked_title'])) {
                    $settings->setAfterBookedTitle($_POST['data']['booked_title']);
                }
                $settings->setBookedEventColor($_POST['data']['booked_color']);
                if (isset($_POST['data']['duration_rule'])) {
                    $settings->setDurationRule($_POST['data']['duration_rule']);
                }
                if (isset($_POST['data']['default_duration'])) {
                    $settings->setFixedDuration($_POST['data']['default_duration']['hours'] * HOUR_IN_SECONDS + $_POST['data']['default_duration']['minutes'] * MINUTE_IN_SECONDS);
                }
                if (isset($_POST['data']['buffer'])) {
                    $settings->setBufferDuration($_POST['data']['buffer']['hours'] * HOUR_IN_SECONDS + $_POST['data']['buffer']['minutes'] * MINUTE_IN_SECONDS);
                }
                $settings->setLinkedEventTitle($_POST['data']['linked_event_title']);
                $settings->setMinTime($_POST['data']['min_time']);
                $settings->setOpenTime($_POST['data']['open_time']);
                if ($_POST['data']['min_time_reference'] === 'end') {
                    $settings->setMinTimeReferenceEnd();
                } else {
                    $settings->setMinTimeReferenceStart();
                }
                $settings->setReminder($_POST['data']['reminder']);
                $settings->setEventDescriptionContent((int)$_POST['data']['event_description_content']);
                $settings->setDealWithUnrelatedEvents((int)$_POST['data']['deal_with_unrelated_events']);
                $settings->setDealWithBookedOfSameService((int)$_POST['data']['deal_with_booked_same']);
                $settings->setDealWithBookedOfOtherServices((int)$_POST['data']['deal_with_booked_others']);
                $location = admin_url('admin.php?page=team-booking&tab=events&gcal=1&event=' . $_POST['service_id'] . '&event_updated=1');
            }
        }
        $coworker_data->setCustomEventSettings($settings, $_POST['service_id']);
        /**
         * We're using the update method, even if it's an
         * object, because if the settings are created just now,
         * then won't be saved (can't be used by reference)
         */
        Functions\getSettings()->updateCoworkerData($coworker_data);
        Functions\getSettings()->save();

        return $location;
    }

    private function saveFrontendStyle()
    {
        if (empty($_POST['tb-background-color'])) {
            $_POST['tb-background-color'] = '#FFFFFF';
        }
        if (empty($_POST['tb-weekline-color'])) {
            $_POST['tb-weekline-color'] = '#FFFFFF';
        }
        if (empty($_POST['tb-freeslot-color'])) {
            $_POST['tb-freeslot-color'] = '#FFFFFF';
        }
        if (empty($_POST['tb-soldoutslot-color'])) {
            $_POST['tb-freeslot-color'] = '#FFFFFF';
        }
        if ($_POST['tb-group-slots-by'] === 'bytime') {
            Functions\getSettings()->setGroupSlotsByTime();
        }
        if ($_POST['tb-group-slots-by'] === 'bycoworker') {
            Functions\getSettings()->setGroupSlotsByCoworker();
        }
        if ($_POST['tb-group-slots-by'] === 'byservice') {
            Functions\getSettings()->setGroupSlotsByService();
        }
        Functions\getSettings()->setColorBackground($_POST['tb-background-color']);
        Functions\getSettings()->setColorWeekLine($_POST['tb-weekline-color']);
        if (isset($_POST['tb-weekline-pattern'])) {
            Functions\getSettings()->setPatternWeekline($_POST['tb-weekline-pattern']);
        }
        if (isset($_POST['tb-calendar-pattern'])) {
            Functions\getSettings()->setPatternCalendar($_POST['tb-calendar-pattern']);
        }
        if (isset($_POST['tb-border-size'])) {
            Functions\getSettings()->setBorderSize($_POST['tb-border-size']);
        }
        if (isset($_POST['tb-border-radius'])) {
            Functions\getSettings()->setBorderRadius($_POST['tb-border-radius']);
        }
        if (isset($_POST['tb-border-color'])) {
            Functions\getSettings()->setBorderColor($_POST['tb-border-color']);
        }
        if (isset($_POST['tb-css-fix'])) {
            Functions\getSettings()->setFix62dot5($_POST['tb-css-fix']);
        }
        Functions\getSettings()->setMapStyle($_POST['tb-map-style']);
        Functions\getSettings()->setGmapsZoomLevel($_POST['tb-map-zoom']);
        Functions\getSettings()->setColorFreeSlot($_POST['tb-freeslot-color']);
        Functions\getSettings()->setColorSoldoutSlot($_POST['tb-soldoutslot-color']);
        Functions\getSettings()->setPriceTagColor($_POST['tb-price-tag-color']);
        Functions\getSettings()->setNumberedDotsLogic($_POST['tb-numbered-dots-logic']);
        Functions\getSettings()->setNumberedDotsLowerBound($_POST['tb-numbered-dots-lower-bound']);
        Functions\getSettings()->save();

        return admin_url('admin.php?page=team-booking&tab=aspect&updated=1');
    }

    private function createApiToken()
    {
        if (isset($_POST['write'])) {
            Functions\getSettings()->addToken(TRUE);
        } else {
            Functions\getSettings()->addToken();
        }
        Functions\getSettings()->save();

        return admin_url('admin.php?page=team-booking&tab=general&token_added=1');
    }

    private function revokeApiToken()
    {
        Functions\getSettings()->revokeToken($_POST['token']);
        Functions\getSettings()->save();

        return admin_url('admin.php?page=team-booking&tab=general&token_removed=1');
    }

    private function resetEnumerableReservations()
    {
        $reservations = Database\Reservations::getByService($_POST['service_id']);
        foreach ($reservations as $reservation) {
            if ($reservation->isEnumerableForCustomerLimits()
                && $reservation->getCustomerUserId() == $_POST['customer_id']
            ) {
                $reservation->setEnumerableForCustomerLimits(FALSE);
                Database\Reservations::update($reservation);
            }
        }

        return 'ok';
    }

    private function addServiceCustomField()
    {
        $service = Functions\getSettings()->getService($_POST['service_id']);
        $service_fields = $service->getFormFields();
        switch ($_POST['field']) {
            case 'text':
                $field = new \TeamBookingFormTextField();
                $field->setLabel(__('A new custom text field', 'team-booking'));
                $hook = $_POST['service_id'] . '_custom_text';
                $i = 0;
                while ($service_fields->isHookDuplicate($hook)) {
                    $i++;
                    $hook .= '_' . $i;
                }
                $field->setHook($hook);
                break;
            case 'textarea':
                $field = new \TeamBookingFormTextarea();
                $field->setLabel(__('A new custom textarea', 'team-booking'));
                $hook = $_POST['service_id'] . '_custom_textarea';
                $i = 0;
                while ($service_fields->isHookDuplicate($hook)) {
                    $i++;
                    $hook .= '_' . $i;
                }
                $field->setHook($hook);
                break;
            case 'select':
                $field = new \TeamBookingFormSelect();
                $field->setLabel(__('A new custom select', 'team-booking'));
                $hook = $_POST['service_id'] . '_custom_select';
                $i = 0;
                while ($service_fields->isHookDuplicate($hook)) {
                    $i++;
                    $hook .= '_' . $i;
                }
                $field->setHook($hook);
                $field->setValue(__('Option One', 'team-booking'));
                $default_option = new \TeamBooking_Components_Form_Option(__('Option Two', 'team-booking'));
                $field->addOption(clone $default_option);
                $default_option->setText(__('Option Three', 'team-booking'));
                $field->addOption(clone $default_option);
                break;
            case 'checkbox':
                $field = new \TeamBookingFormCheckbox();
                $field->setLabel(__('A new custom checkbox', 'team-booking'));
                $hook = $_POST['service_id'] . '_custom_checkbox';
                $i = 0;
                while ($service_fields->isHookDuplicate($hook)) {
                    $i++;
                    $hook .= '_' . $i;
                }
                $field->setHook($hook);
                $field->setCheckedOff();
                $field->setValue($field->getLabel());
                break;
            case 'radio':
                $field = new \TeamBookingFormRadio();
                $field->setLabel(__('A new custom radio group', 'team-booking'));
                $hook = $_POST['service_id'] . '_custom_radio';
                $i = 0;
                while ($service_fields->isHookDuplicate($hook)) {
                    $i++;
                    $hook .= '_' . $i;
                }
                $field->setHook($hook);
                $field->setValue(__('Option One', 'team-booking'));
                $default_option = new \TeamBooking_Components_Form_Option(__('Option Two', 'team-booking'));
                $field->addOption(clone $default_option);
                $default_option->setText(__('Option Three', 'team-booking'));
                $field->addOption(clone $default_option);
                break;
            case 'file':
                $field = new \TeamBookingFormFileUpload();
                $field->setLabel(__('A new file upload field', 'team-booking'));
                $hook = $_POST['service_id'] . '_custom_file';
                $i = 0;
                while ($service_fields->isHookDuplicate($hook)) {
                    $i++;
                    $hook .= '_' . $i;
                }
                $field->setHook($hook);
                break;
            default :
                return TRUE;
        }
        $field->setOn();
        $field->setRequiredOff();
        // Extreme check to avoid corruptions
        if (NULL !== $field) {
            $service_fields->addCustomField($field);
            $service->setFormFields($service_fields);
            Functions\getSettings()->updateService($service, $service->getId());
            Functions\getSettings()->save();

            return '<li>' . Mappers\adminFormCustomFieldsMapper($field, $service->getId()) . '</li>';
        }

        return TRUE;
    }

    private function removeServiceCustomField()
    {
        $service = Functions\getSettings()->getService($_POST['service_id']);
        $fields = $service->getFormFields();
        $fields->dropCustomField($_POST['hook']);
        $service->setFormFields($fields);
        Functions\getSettings()->updateService($service, $service->getId());
        Functions\getSettings()->save();

        return 'ok';
    }

    private function saveServiceCustomField()
    {
        $service = Functions\getSettings()->getService($_POST['service_id']);
        $fields = $service->getFormFields();
        $custom_fields = $fields->getCustomFields();
        foreach ($custom_fields as $custom_field) {
            if ($custom_field->getHook() == $_POST['hook']) {
                $field = $custom_field;
            }
        }
        // Parse data
        $parsed_data = array();
        parse_str($_POST['inputs'], $parsed_data);
        $parsed_data = $parsed_data['event']['form_fields']['custom'];
        // Checking Hook availability, if new
        if ($_POST['hook'] != $parsed_data[ $_POST['hook'] . '_hook' ]) {
            if ($fields->isHookDuplicate($parsed_data[ $_POST['hook'] . '_hook' ])) {
                return 'duplicate_hook';
            }
        }
        // Sets data
        /* @var $field \TeamBookingFormTextField */
        $field->setHook($parsed_data[ $_POST['hook'] . '_hook' ]);
        $field->setLabel($parsed_data[ $_POST['hook'] . '_label' ]);
        if (isset($parsed_data[ $_POST['hook'] . '_show' ])) {
            $field->setOn();
        } else {
            $field->setOff();
        }
        if (isset($parsed_data[ $_POST['hook'] . '_extensions' ])) {
            $field->setFileExtensions($parsed_data[ $_POST['hook'] . '_extensions' ]);
        }
        if (isset($parsed_data[ $_POST['hook'] . '_required' ])) {
            $field->setRequiredOn();
        } else {
            $field->setRequiredOff();
        }
        if (get_class($field) === 'TeamBookingFormCheckbox') {
            if (isset($parsed_data[ $_POST['hook'] . '_selected' ])) {
                $field->setCheckedOn();
            } else {
                $field->setCheckedOff();
            }
        }
        if (isset($parsed_data[ $_POST['hook'] . '_default' ])) {
            if (isset($parsed_data[ $_POST['hook'] . '_default' ]['label'])) {
                $obj = new \TeamBooking_Components_Form_Option();
                $obj->setText($parsed_data[ $_POST['hook'] . '_default' ]['label']);
                $obj->setPriceIncrement($parsed_data[ $_POST['hook'] . '_default' ]['price_increment']);
                $field->setValue($obj);
            } else {
                $field->setValue($parsed_data[ $_POST['hook'] . '_default' ]);
            }
        }
        if (isset($parsed_data[ $_POST['hook'] . '_options' ])) {
            $field->resetOptions();
            foreach ($parsed_data[ $_POST['hook'] . '_options' ] as $option) {
                $obj = new \TeamBooking_Components_Form_Option();
                $obj->setText($option['label']);
                $obj->setPriceIncrement($option['price_increment']);
                $field->addOption($obj);
            }
        }
        if (isset($parsed_data[ $_POST['hook'] . '_regex' ])) {
            $field->setValidationRegex($parsed_data[ $_POST['hook'] . '_regex' ]);
        }
        if (isset($parsed_data[ $_POST['hook'] . '_validation' ])) {
            $field->setValidationRule($parsed_data[ $_POST['hook'] . '_validation' ]);
        }
        // Saving
        $fields->updateCustomField($_POST['hook'], $field);
        $service->setFormFields($fields);
        Functions\getSettings()->updateService($service, $service->getId());
        Functions\getSettings()->save();

        return 'ok';
    }

    private function moveServiceCustomField()
    {
        $service = Functions\getSettings()->getService($_POST['service_id']);
        $custom_fields = $service->getFormFields()->getCustomFields();
        foreach ($custom_fields as $key => $field) {
            if ($field->getHook() == $_POST['hook']) {
                $moved_field = $field;
                $service->getFormFields()->dropCustomField($_POST['hook']);
                break;
            }
        }
        $custom_fields = $service->getFormFields()->getCustomFields();
        array_splice($custom_fields, $_POST['where'], 0, array($moved_field));
        $service->getFormFields()->saveCustomFields($custom_fields);
        Functions\getSettings()->save();

        return 'ok';
    }

    private function toggleServiceActivation()
    {
        if (isset($_POST['personal'])) {
            $coworker_data = Functions\getSettings()->getCoworkerData(get_current_user_id());
            $custom_settings = $coworker_data->getCustomEventSettings($_POST['service_id']);
            $custom_settings->setParticipate($_POST['service_action'] === 'activate');
            $coworker_data->setCustomEventSettings($custom_settings, $_POST['service_id']);
            Functions\getSettings()->save();
        } else {
            $service = Functions\getSettings()->getService($_POST['service_id']);
            $service->setActive($_POST['service_action'] === 'activate');
            Functions\getSettings()->save();
        }

        return 'ok';
    }

    private function toggleCalendarIndependency()
    {
        $coworker_data = Functions\getSettings()->getCoworkerData(get_current_user_id());
        $calendars = $coworker_data->getCalendars();
        $calendars[ $_POST['calendar_id'] ]['independent'] = ((bool)$_POST['independent']);
        $coworker_data->addCalendars($calendars);
        Functions\getSettings()->updateCoworkerData($coworker_data);
        Functions\getSettings()->save();

        return 'ok';
    }

    private function cloneService()
    {
        $new_service_id = Toolkit\filterInput($_POST['new_service_id'], TRUE);
        if (empty($new_service_id) || Functions\checkServiceIdExistance($new_service_id)) {
            return 'id already used';
        } else {
            $service_id = $_POST['service_id'];
            // Can't use clone() as it will produce shallow copies...
            $service = unserialize(serialize(Functions\getSettings()->getService($service_id)));
            $service->setId($new_service_id);
            $i = 1;
            $service_name = $service->getName();
            while (Functions\checkServiceNameExistance($service_name)) {
                $i++;
                $service_name = $service->getName() . '-' . $i;
            }
            $service->setName($service_name);
            $service->save();

            return $service_id;
        }
    }

    private function cloneServiceSuggestion()
    {
        $service_id = $_POST['service_id'];
        $service = Functions\getSettings()->getService($service_id);
        $i = 1;
        while (Functions\checkServiceIdExistance($service_id)) {
            $i++;
            $service_id = $service->getId() . '-' . $i;
        }

        return $service_id;
    }

    private function insertPromotionCampaign()
    {
        $services_array = $_POST['campaign_services'];
        $name = Toolkit\filterInput($_POST['campaign_name'], TRUE);
        $start_date = \DateTime::createFromFormat('m/d/Y|', $_POST['campaign_start']);
        $end_date = \DateTime::createFromFormat('m/d/Y|', $_POST['campaign_end']);
        $errors = '';
        if (!$start_date || $start_date->format('m/d/Y') != $_POST['campaign_start']) {
            $errors .= 'start_date_fail';
        }
        if (!$end_date || $end_date->format('m/d/Y') != $_POST['campaign_end']) {
            $errors .= 'end_date_fail';
        }
        if (count(Database\Promotions::getByName($name)) > 0) {
            $errors .= 'name_fail';
        }

        if (!empty($errors)) {
            return $errors;
        }

        $new_campaign = new \TeamBooking_Promotions_Campaign();
        $new_campaign->addServices($services_array);
        $new_campaign->setName($name);
        if (filter_input(INPUT_POST, 'campaign_discount', FILTER_VALIDATE_INT) > 100
            && filter_input(INPUT_POST, 'campaign_discount_type') === 'percentage'
        ) {
            $new_campaign->setDiscount(100);
        } else {
            $new_campaign->setDiscount(filter_input(INPUT_POST, 'campaign_discount', FILTER_VALIDATE_INT));
        }
        $new_campaign->setDiscountType(filter_input(INPUT_POST, 'campaign_discount_type'));
        $new_campaign->setStartTime($start_date->getTimestamp());
        $new_campaign->setEndTime($end_date->getTimestamp());
        $new_campaign->setStatus(TRUE);
        Database\Promotions::insert($new_campaign);

        return 'ok';
    }

    private function insertPromotionCoupon()
    {
        $services_array = $_POST['coupon_services'];
        $name = Toolkit\filterInput($_POST['coupon_name'], TRUE);
        $start_date = \DateTime::createFromFormat('m/d/Y|', $_POST['coupon_start']);
        $end_date = \DateTime::createFromFormat('m/d/Y|', $_POST['coupon_end']);
        $errors = '';
        if (!$start_date || $start_date->format('m/d/Y') != $_POST['coupon_start']) {
            $errors .= 'start_date_fail';
        }
        if (!$end_date || $end_date->format('m/d/Y') != $_POST['coupon_end']) {
            $errors .= 'end_date_fail';
        }
        if (count(Database\Promotions::getByName($name)) > 0) {
            $errors .= 'name_fail';
        }

        if (!empty($errors)) {
            return $errors;
        }

        $new_coupon = new \TeamBooking_Promotions_Coupon();
        $new_coupon->addServices($services_array);
        $new_coupon->setName($name);
        if (filter_input(INPUT_POST, 'coupon_discount', FILTER_VALIDATE_INT) > 100
            && filter_input(INPUT_POST, 'coupon_discount_type') === 'percentage'
        ) {
            $new_coupon->setDiscount(100);
        } else {
            $new_coupon->setDiscount(filter_input(INPUT_POST, 'coupon_discount', FILTER_VALIDATE_INT));
        }
        $new_coupon->setDiscountType(filter_input(INPUT_POST, 'coupon_discount_type'));
        $new_coupon->setStartTime($start_date->getTimestamp());
        $new_coupon->setEndTime($end_date->getTimestamp());
        $new_coupon->setStatus(TRUE);
        Database\Promotions::insert($new_coupon);

        return 'ok';
    }

    private function changePromotionStatus()
    {
        $promotion = Database\Promotions::getById($_POST['pricing_id']);
        if ($_POST['status'] === 'run') {
            $promotion->setStatus(TRUE);
        } else {
            $promotion->setStatus(FALSE);
        }
        Database\Promotions::update($_POST['pricing_id'], $promotion);

        return 'ok';
    }

    private function deletePromotion()
    {
        Database\Promotions::delete($_POST['pricing_id']);

        return 'ok';
    }

    private function editPromotion()
    {
        $services_array = $_POST['pricing_services'];
        $promotion = Database\Promotions::getById($_POST['pricing_db_id']);
        $name = Toolkit\filterInput($_POST['pricing_name'], TRUE);
        $start_date = \DateTime::createFromFormat('m/d/Y|', $_POST['pricing_start']);
        $end_date = \DateTime::createFromFormat('m/d/Y|', $_POST['pricing_end']);
        $errors = '';
        if (!$start_date || $start_date->format('m/d/Y') != $_POST['pricing_start']) {
            $errors .= 'start_date_fail';
        }
        if (!$end_date || $end_date->format('m/d/Y') != $_POST['pricing_end']) {
            $errors .= 'end_date_fail';
        }
        if ($name != $promotion->getName()
            && count(Database\Promotions::getByName($name)) > 0
        ) {
            $errors .= 'name_fail';
        }

        if (!empty($errors)) {
            return $errors;
        }

        $promotion->addServices($services_array);
        $promotion->setName($name);
        if (filter_input(INPUT_POST, 'pricing_discount', FILTER_VALIDATE_INT) > 100
            && filter_input(INPUT_POST, 'pricing_discount_type') === 'percentage'
        ) {
            $promotion->setDiscount(100);
        } else {
            $promotion->setDiscount(filter_input(INPUT_POST, 'pricing_discount', FILTER_VALIDATE_INT));
        }
        $promotion->setDiscountType(filter_input(INPUT_POST, 'pricing_discount_type'));
        $promotion->setStartTime($start_date->getTimestamp());
        $promotion->setEndTime($end_date->getTimestamp());
        Database\Promotions::update($_POST['pricing_db_id'], $promotion);

        return 'ok';
    }

    public function addTinyMCEButton()
    {
        global $typenow;
        // check user permissions
        if (!current_user_can('edit_posts') && !current_user_can('edit_pages')) {
            return;
        }
        // verify the post type
        if (!in_array($typenow, array(
            'post',
            'page',
        ))
        )
            return;
        // check if WYSIWYG is enabled
        if (get_user_option('rich_editing') === 'true') {
            add_filter('mce_external_plugins', array(
                $this,
                'addTinyMCEPlugin',
            ));
            add_filter('mce_buttons', array(
                $this,
                'registerTinyMCEButton',
            ));
        }
    }

    public function addTinyMCEPlugin($plugin_array)
    {
        $plugin_array['teambooking_tinymce_button'] = TEAMBOOKING_URL . 'js/tinymce_button.js';

        return $plugin_array;
    }

    public function registerTinyMCEButton($buttons)
    {
        $buttons[] = 'teambooking_tinymce_button';

        return $buttons;
    }

}
