<?php

defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Toolkit;

class TeamBookingType
{
    private $booking_name;
    private $booking_id;
    private $booking_class;
    private $service_color;
    private $email_for_notifications;
    private $notify;
    private $confirm;
    private $logged_only;
    private $logged_max_reservations;
    private $show_times;
    private $show_soldout;
    private $show_coworker;
    private $show_coworker_url;
    private $attendees;
    private $max_reservations_per_user;
    private $show_reservations_left;
    private $service_info;
    private $price;
    private $active;
    private $coworker_assignment_rule;
    private $coworker_id_for_direct_assignment;
    private $payment_must_be_done;
    private $include_files_as_attachment; // To admin notification address
    private $duration_rule;
    private $default_duration;
    private $send_cancellation_email; // To customer
    private $send_cancellation_email_backend; // To admin/coworker
    private $send_reminder_email;
    private $location_setting;
    private $location_address;
    private $approve_rule;
    private $free_until_approval;
    private $allow_customer_cancellation;
    private $allow_customer_cancellation_reason;
    private $allow_customer_cancellation_until; //seconds
    private $redirect;
    private $redirect_url;

    /** @var $cancellation_email TeamBookingServiceEmail */
    private $cancellation_email;

    /** @var $cancellation_email_backend TeamBookingServiceEmail */
    private $cancellation_email_backend;

    /** @var $front_end_email TeamBookingServiceEmail */
    private $front_end_email;

    /** @var $back_end_email TeamBookingServiceEmail */
    private $back_end_email;

    /** @var $reminder_email TeamBookingServiceEmail */
    private $reminder_email;

    /** @var $form_fields TeamBookingFormFields */
    private $form_fields;

    public function __construct()
    {
        $this->setConfirm(TRUE);
        $this->setNotify(TRUE);
        $this->setSendCancellationEmail(TRUE);
        $this->email_for_notifications = get_bloginfo('admin_email');
        $this->front_end_email = new TeamBookingServiceEmail();
        $this->back_end_email = new TeamBookingServiceEmail();
        $this->cancellation_email = new TeamBookingServiceEmail();
        $this->reminder_email = new TeamBookingServiceEmail();
        $this->send_reminder_email = array();
        // Default email content
        $this->front_end_email->setSubject(esc_html__('Your reservation details', 'team-booking'));
        $default_frontend_body = esc_html__('Thanks for your reservation!', 'team-booking') . '<br>'
            . esc_html__('Reservation date and time', 'team-booking') . ': [start_datetime]';
        $this->front_end_email->setBody($default_frontend_body);
        $this->back_end_email->setSubject(esc_html__('A new reservation', 'team-booking'));
        $default_backend_body = esc_html__('You have just got a new reservation!', 'team-booking') . '<br>'
            . esc_html__('Customer name', 'team-booking') . ': [first_name]<br>'
            . esc_html__('Customer email', 'team-booking') . ': [email]';
        $this->back_end_email->setBody($default_backend_body);
        $this->cancellation_email->setSubject(esc_html__('Reservation cancelled', 'team-booking'));
        $default_cancellation_body = esc_html__('Your reservation was cancelled.', 'team-booking');
        $this->cancellation_email->setBody($default_cancellation_body);
        $this->reminder_email->setSubject(esc_html__("Don't forget your reservation", 'team-booking'));
        $default_reminder_body = esc_html__('We are getting close!', 'team-booking') . '<br>'
            . esc_html__('Reservation date and time', 'team-booking') . ': [start_datetime]';
        $this->reminder_email->setBody($default_reminder_body);
        $this->setLoggedOnly(FALSE);
        $this->setShowTimes(new TeamBookingShowTimes);
        $this->setShowCoworker(FALSE);
        $this->setShowCoworkerUrl(FALSE);
        $this->setShowSoldout(FALSE);
        $this->setPrice(0);
        $this->setActive(TRUE);
        $this->setPaymentMustBeDone('immediately');
        $this->setIncludeFilesAsAttachment(FALSE);
        $this->setDurationRule('coworker');
        $this->setApproveRule('none');
        $this->setDefaultDuration(1 * HOUR_IN_SECONDS);
        // Set a random color
        $this->service_color = sprintf('#%06X', mt_rand(0, 0xFFFFFF));
        $this->location_address = '';
        $this->free_until_approval = FALSE;
        $this->allow_customer_cancellation = TRUE;
        $this->allow_customer_cancellation_reason = TRUE;
        $this->allow_customer_cancellation_until = 1 * DAY_IN_SECONDS;
        $this->redirect = FALSE;
        $this->redirect_url = '';
        $this->logged_max_reservations = 0; // 0 = infinite
    }

    //------------------------------------------------------------

    public function setConfirm($bool)
    {
        $this->confirm = (bool)$bool;
    }

    public function getConfirm()
    {
        return $this->confirm;
    }

    public function setNotify($bool)
    {
        $this->notify = (bool)$bool;
    }

    public function getNotify()
    {
        return $this->notify;
    }

    public function setSendCancellationEmail($bool)
    {
        $this->send_cancellation_email = (bool)$bool;
    }

    public function getSendCancellationEmail()
    {
        return isset($this->send_cancellation_email) ? $this->send_cancellation_email : TRUE;
    }

    public function setSendCancellationEmailBackend($bool)
    {
        $this->send_cancellation_email_backend = (bool)$bool;
    }

    public function getSendCancellationEmailBackend()
    {
        return isset($this->send_cancellation_email_backend) ? $this->send_cancellation_email_backend : TRUE;
    }

    public function setActive($bool)
    {
        $this->active = (bool)$bool;
    }

    public function isActive()
    {
        return isset($this->active) ? $this->active : TRUE;
    }

    public function setIncludeFilesAsAttachment($bool)
    {
        $this->include_files_as_attachment = (bool)$bool;
    }

    public function getIncludeFilesAsAttachment()
    {
        return isset($this->include_files_as_attachment) ? $this->include_files_as_attachment : FALSE;
    }

    public function setName($name)
    {
        $this->booking_name = Toolkit\filterInput($name);
    }

    public function getName()
    {
        return Toolkit\unfilterInput($this->booking_name);
    }

    public function setId($id)
    {
        $this->booking_id = Toolkit\filterInput($id, TRUE);
    }

    public function getId()
    {
        return $this->booking_id;
    }

    public function getLocationSetting()
    {
        if (!$this->location_setting) {
            return ($this->booking_class == 'service') ? 'none' : 'inherited';
        } else {
            return $this->location_setting;
        }
    }

    public function setLocationSetting($setting)
    {
        if ($setting == 'none') {
            $this->location_setting = 'none';
        } elseif ($setting == 'inherited') {
            $this->location_setting = 'inherited';
        } else {
            $this->location_setting = 'fixed';
        }
    }

    public function getLocationAddress()
    {
        return isset($this->location_address) ? $this->location_address : '';
    }

    public function setLocationAddress($address)
    {
        $this->location_address = $address;
    }

    public function getApproveRule()
    {
        return (!$this->approve_rule || $this->getPaymentMustBeDone() !== 'later') ? 'none' : $this->approve_rule;
    }

    public function setApproveRule($rule)
    {
        if ($rule === 'admin' || $rule === 'coworker') {
            $this->approve_rule = $rule;
        } else {
            $this->approve_rule = 'none';
        }
    }

    public function getPaymentMustBeDone()
    {
        return isset($this->payment_must_be_done) ? $this->payment_must_be_done : 'immediately';
    }

    public function setPaymentMustBeDone($how)
    {
        $this->payment_must_be_done = $how;
    }

    public function setFreeUntilApproval($boolean)
    {
        $this->free_until_approval = (bool)$boolean;
    }

    public function getFreeUntilApproval()
    {
        return isset($this->free_until_approval) ? $this->free_until_approval : FALSE;
    }

    public function setAllowCustomerCancellation($boolean)
    {
        $this->allow_customer_cancellation = (bool)$boolean;
    }

    public function getAllowCustomerCancellation()
    {
        return isset($this->allow_customer_cancellation) ? $this->allow_customer_cancellation : FALSE;
    }

    public function setAllowCustomerCancellationReason($boolean)
    {
        $this->allow_customer_cancellation_reason = (bool)$boolean;
    }

    public function getAllowCustomerCancellationReason()
    {
        return isset($this->allow_customer_cancellation_reason) ? $this->allow_customer_cancellation_reason : TRUE;
    }

    public function setAllowCustomerCancellationUntil($seconds)
    {
        $this->allow_customer_cancellation_until = (int)$seconds;
    }

    public function getAllowCustomerCancellationUntil()
    {
        return isset($this->allow_customer_cancellation_until) ? $this->allow_customer_cancellation_until : (1 * DAY_IN_SECONDS);
    }

    public function setRedirect($value)
    {
        $this->redirect = $value === 'no_redirect' ? FALSE : TRUE;
    }

    public function getRedirect()
    {
        return isset($this->redirect) ? $this->redirect : FALSE;
    }

    public function setRedirectUrl($url)
    {
        $this->redirect_url = $url;
    }

    public function getRedirectUrl($reservation_id = NULL)
    {
        if (isset($this->redirect_url)) {
            if (is_null($reservation_id)) {
                return $this->redirect_url;
            } else {
                $url = parse_url($this->redirect_url);
                $params = array();
                if (isset($url['query'])) {
                    parse_str($url['query'], $params);
                }
                $params['reservation_database_id'] = $reservation_id;
                $redirect = $url['scheme'] . '://';
                if (isset($url['host'])) $redirect .= $url['host'];
                if (isset($url['path'])) $redirect .= $url['path'];
                if (!empty($params)) $redirect .= '?' . http_build_query($params);

                return $redirect;
            }
        } else {
            return '';
        }
    }

    public function getServiceColor()
    {
        return isset($this->service_color) ? $this->service_color : '#e07b53'; // Orange
    }

    public function setServiceColor($color)
    {
        $this->service_color = $color;
    }

    public function getDurationRule()
    {
        return isset($this->duration_rule) ? $this->duration_rule : 'coworker';
    }

    public function setDurationRule($rule)
    {
        $this->duration_rule = $rule;
    }

    public function getDefaultDuration()
    {
        return (isset($this->default_duration) && $this->default_duration > 0) ? $this->default_duration : 3600;
    }

    public function setDefaultDuration($seconds)
    {
        $this->default_duration = $seconds;
    }

    public function setClass($class)
    {
        $this->booking_class = $class;
        if ($this->booking_class == 'event') {
            $this->form_fields = new TeamBookingFormFields();
            $this->attendees = 10;
            $this->max_reservations_per_user = 2;
            $this->show_reservations_left = TRUE;
        } elseif ($this->booking_class == 'appointment' || $this->booking_class == 'service') {
            $this->form_fields = new TeamBookingFormFields();
            $this->attendees = NULL;
            $this->show_reservations_left = FALSE;
        }
        if ($this->booking_class == 'service') {
            $this->coworker_assignment_rule = 'equal';
            #$this->coworker_assignment_rule = "direct";
            #$this->coworker_assignment_rule = "random";
        }
    }

    public function getClass($as_strings = FALSE)
    {
        if (!$as_strings) {
            return $this->booking_class;
        } else {
            switch ($this->booking_class) {
                case 'appointment':
                    return __('Appointment', 'team-booking');
                    break;
                case 'event':
                    return __('Event', 'team-booking');
                    break;
                case 'service':
                    return __('Unscheduled', 'team-booking');
                    break;
                default:
                    return $this->booking_class;
                    break;
            }
        }
    }

    public function isClass($class_name)
    {
        return (strtolower($this->booking_class) === strtolower($class_name));
    }

    public function setLoggedOnly($bool)
    {
        $this->logged_only = (bool)$bool;
    }

    public function getLoggedOnly()
    {
        return $this->logged_only;
    }

    public function setShowReservationsLeft($bool)
    {
        $this->show_reservations_left = (bool)$bool;
    }

    public function getShowReservationsLeft()
    {
        return $this->show_reservations_left;
    }

    public function setShowSoldout($bool)
    {
        $this->show_soldout = (bool)$bool;
    }

    public function getShowSoldout()
    {
        return $this->show_soldout;
    }

    public function setShowCoworker($bool)
    {
        $this->show_coworker = (bool)$bool;
    }

    public function getShowCoworker()
    {
        return $this->show_coworker;
    }

    public function setShowCoworkerUrl($bool)
    {
        $this->show_coworker_url = (bool)$bool;
    }

    public function getShowCoworkerUrl()
    {
        return isset($this->show_coworker_url) ? $this->show_coworker_url : FALSE;
    }

    public function setAssignmentRule($rulename)
    {
        $this->coworker_assignment_rule = $rulename;
    }

    public function getAssignmentRule()
    {
        return ($this->isClass('service') || $this->coworker_assignment_rule === 'direct') ? $this->coworker_assignment_rule : 'none';
    }

    public function setCoworkerForDirectAssignment($coworker_id)
    {
        $this->coworker_id_for_direct_assignment = $coworker_id;
    }

    public function getCoworkerForDirectAssignment()
    {
        return $this->coworker_id_for_direct_assignment;
    }

    public function getEmailForNotifications()
    {
        return $this->email_for_notifications;
    }

    public function setEmailForNotifications($email)
    {
        $this->email_for_notifications = $email;
    }

    public function getAttendees()
    {
        return $this->attendees;
    }

    public function setAttendees($number)
    {
        if ($number > 200) $number = 200;
        $this->attendees = $number;
    }

    public function getMaxReservationsPerUser()
    {
        // Backward compatibility before 1.2.5
        return empty($this->max_reservations_per_user) ? $this->attendees : $this->max_reservations_per_user;
    }

    public function setMaxReservationsPerUser($number)
    {
        $this->max_reservations_per_user = $number;
    }

    public function getMaxReservationsLoggedUser()
    {
        return (!$this->logged_max_reservations || !$this->logged_only) ? 0 : $this->logged_max_reservations;
    }

    public function setMaxReservationsLoggedUser($int)
    {
        $this->logged_max_reservations = (int)$int;
    }

    public function getServiceInfo()
    {
        return Toolkit\unfilterInput($this->service_info);
    }

    public function setServiceInfo($info)
    {
        $this->service_info = Toolkit\filterInput($info);
    }

    /**
     *
     * @return TeamBookingServiceEmail
     */
    public function getFrontendEmail()
    {
        return $this->front_end_email;
    }

    public function setFrontendEmail($email)
    {
        $this->front_end_email = $email;
    }

    /**
     *
     * @return TeamBookingServiceEmail
     */
    public function getBackendEmail()
    {
        return $this->back_end_email;
    }

    public function setBackendEmail($email)
    {
        $this->back_end_email = $email;
    }

    /**
     *
     * @return TeamBookingServiceEmail
     */
    public function getCancellationEmail()
    {
        if (isset($this->cancellation_email)) {
            return $this->cancellation_email;
        } else {
            $return = new TeamBookingServiceEmail();
            $return->setBody(__('Your reservation was cancelled.', 'team-booking'));
            $return->setSubject(__('Reservation cancelled', 'team-booking'));

            return $return;
        }
    }

    public function setCancellationEmail($email)
    {
        $this->cancellation_email = $email;
    }

    /**
     *
     * @return TeamBookingServiceEmail
     */
    public function getCancellationEmailBackend()
    {
        if (isset($this->cancellation_email_backend)) {
            return $this->cancellation_email_backend;
        } else {
            $return = new TeamBookingServiceEmail();
            $return->setBody(__('Your reservation was cancelled.', 'team-booking'));
            $return->setSubject(__('Reservation cancelled', 'team-booking'));

            return $return;
        }
    }

    public function setCancellationEmailBackend($email)
    {
        $this->cancellation_email_backend = $email;
    }

    /**
     *
     * @return TeamBookingServiceEmail
     */
    public function getReminderEmail()
    {
        if (isset($this->reminder_email)) {
            return $this->reminder_email;
        } else {
            $return = new TeamBookingServiceEmail();
            $return->setSubject(esc_html__("Don't forget your reservation", 'team-booking'));
            $default_reminder_body = esc_html__("We're getting close!", 'team-booking') . '<br>'
                . esc_html__('Reservation date and time', 'team-booking') . ': [start_datetime]';
            $return->setBody($default_reminder_body);

            return $return;
        }
    }

    public function setReminderEmail($email)
    {
        $this->reminder_email = $email;
    }

    /**
     * @return array
     */
    public function getSendReminderEmail()
    {
        return isset($this->send_reminder_email) ? $this->send_reminder_email : array();
    }

    public function setSendReminderEmail($array)
    {
        $this->send_reminder_email = $array;
    }

    public function isSendReminderEmail()
    {
        return (count($this->getSendReminderEmail()) < 1) ? FALSE : TRUE;
    }

    /**
     *
     * @return TeamBookingFormFields
     */
    public function getFormFields()
    {
        // legacy
        if (!$this->form_fields->getUrl() instanceof TeamBookingFormTextField) {
            $obj = new TeamBookingFormTextField();
            $obj->setHook('url');
            $obj->setLabel(__('Website', 'team-booking'));
            $this->form_fields->url = $obj;
        }

        return $this->form_fields;
    }

    public function setFormFields($form_fields)
    {
        $this->form_fields = $form_fields;
    }

    /**
     *
     * @return TeamBookingShowTimes
     */
    public function getShowTimes()
    {
        return $this->show_times;
    }

    /**
     *
     * @param TeamBookingShowTimes $object
     */
    public function setShowTimes($object)
    {
        $this->show_times = $object;
    }

    public function getPrice()
    {
        return $this->price;
    }

    public function setPrice($price)
    {
        $this->price = $price;
    }

    public function getPotentialPrice()
    {
        $potential_price = $this->getPrice();
        $increment = 0;
        $fields = $this->getFormFields()->getCustomFields();
        foreach ($fields as $field) {
            if ($field->getValue() instanceof TeamBooking_Components_Form_Option
                && $field->getValue()->getPriceIncrement() > $increment
            ) {
                $increment = $field->getValue()->getPriceIncrement();
            }
        }

        return $potential_price + $increment;
    }

    //------------------------------------------------------------

    public function save()
    {
        $settings = Functions\getSettings();
        $settings->updateService($this, $this->booking_id);

        return $settings->save();
    }

    //------------------------------------------------------------

    public function getApiResource()
    {
        $return = array(
            'type'                         => 'service',
            'id'                           => $this->booking_id,
            'name'                         => $this->booking_name,
            'class'                        => $this->booking_class,
            'color'                        => $this->service_color,
            'description'                  => $this->service_info,
            'isActive'                     => $this->isActive(),
            'notificationEmailAdmin'       => array(
                'send'    => $this->getNotify(),
                'address' => $this->getEmailForNotifications(),
                'subject' => $this->getBackendEmail()->getSubject(),
                'body'    => $this->getBackendEmail()->getBody()
            ),
            'confirmationEmail'            => array(
                'send'    => $this->getConfirm(),
                'subject' => $this->getFrontendEmail()->getSubject(),
                'body'    => $this->getFrontendEmail()->getBody()
            ),
            'reminderEmail'                => array(
                'send'    => $this->getSendReminderEmail(),
                'subject' => $this->getReminderEmail()->getSubject(),
                'body'    => $this->getReminderEmail()->getBody()
            ),
            'approval'                     => array(
                'requireFrom'           => $this->getApproveRule(),
                'keepFreeUntilApproval' => $this->getFreeUntilApproval()
            ),
            'cancellationByCustomer'       => array(
                'allow'    => $this->getAllowCustomerCancellation(),
                'timespan' => $this->getAllowCustomerCancellationUntil()
            ),
            'payments'                     => array(
                'price'    => $this->getPrice(),
                'required' => $this->getPaymentMustBeDone()
            ),
            'loggedOnly'                   => $this->getLoggedOnly(),
            'maxTotalTicketsPerSlot'       => $this->getAttendees(),
            'maxUserTicketsPerSlot'        => $this->getMaxReservationsPerUser(),
            'maxReservationsPerLoggedUser' => $this->getMaxReservationsLoggedUser(),
            'location'                     => array(
                'setting' => $this->getLocationSetting(),
                'address' => $this->getLocationAddress()
            ),
            'redirectURL'                  => $this->getRedirectUrl(),
            'assignment'                   => array(
                'rule'             => $this->getAssignmentRule(),
                'directCoworkerID' => $this->getCoworkerForDirectAssignment()
            )
        );
        if ($this->isClass('service')) $return['reminderEmail'] = NULL;
        if ($this->isClass('service')) $return['maxTotalTicketsPerSlot'] = NULL;
        if ($this->isClass('service')) $return['maxUserTicketsPerSlot'] = NULL;

        return $return;
    }

}

/**
 * Just an object where are stored the frontend show times options
 */
class TeamBookingShowTimes
{
    private $yes; // Start + end (or All day)
    private $no;  // Nothing
    private $start_time_only; // Start only (or All day)

    public function __construct()
    {
        $this->yes = TRUE;
    }

    //------------------------------------------------------------

    public function setValue($var_name)
    {
        switch ($var_name) {
            case 'yes':
                $this->setYes();
                break;
            case 'no':
                $this->setNo();
                break;
            case 'start_time_only':
                $this->setStartOnly();
                break;
            default:
                return FALSE;
        }

        return TRUE;
    }

    private function setYes()
    {
        $this->yes = TRUE;
        $this->no = FALSE;
        $this->start_time_only = FALSE;
    }

    private function setNo()
    {
        $this->yes = FALSE;
        $this->no = TRUE;
        $this->start_time_only = FALSE;
    }

    private function setStartOnly()
    {
        $this->yes = FALSE;
        $this->no = FALSE;
        $this->start_time_only = TRUE;
    }

    public function getValue()
    {
        foreach (get_object_vars($this) as $var => $value) {
            if ($value) {
                return $var;
            }
        }

        return NULL;
    }

    public function getOptionsList()
    {
        return get_object_vars($this);
    }

    public function getOptionDescription($var_name)
    {
        switch ($var_name) {
            case 'yes':
                return __('Show start/end', 'team-booking');
            case 'no':
                return __("Don't show", 'team-booking');
            case 'start_time_only':
                return __('Show start time only', 'team-booking');
            default:
                return NULL;
        }
    }

}

/**
 * Class TeamBookingServiceEmail
 */
class TeamBookingServiceEmail
{

    //------------------------------------------------------------

    private $subject;
    private $body;

    //------------------------------------------------------------

    public function getSubject()
    {
        return Toolkit\unfilterInput($this->subject);
    }

    public function setSubject($text)
    {
        $this->subject = Toolkit\filterInput($text);
    }

    public function getBody()
    {
        return Toolkit\unfilterInput($this->body);
    }

    public function setBody($text)
    {
        $this->body = Toolkit\filterInput($text);
    }

}

/**
 * Class TeamBookingFormFields
 */
class TeamBookingFormFields
{

    //------------------------------------------------------------

    /** @var TeamBookingFormTextField */
    public $first_name;

    /** @var TeamBookingFormTextField */
    public $second_name;

    /** @var TeamBookingFormTextField */
    public $email;

    /** @var TeamBookingFormTextField */
    public $address;

    /** @var TeamBookingFormTextField */
    public $phone;

    /** @var TeamBookingFormTextField */
    public $url;

    /** @var TeamBookingFormTextField[] */
    public $custom_fields = array();

    public function __construct()
    {
        $this->first_name = new TeamBookingFormTextField();
        $this->second_name = new TeamBookingFormTextField();
        $this->email = new TeamBookingFormTextField();
        $this->address = new TeamBookingFormTextField();
        $this->phone = new TeamBookingFormTextField();
        $this->url = new TeamBookingFormTextField();
        $this->first_name->setHook('first_name');
        $this->first_name->setLabel(__('First name', 'team-booking'));
        $this->second_name->setHook('second_name');
        $this->second_name->setLabel(__('Last name', 'team-booking'));
        $this->email->setHook('email');
        $this->email->setLabel(__('Email', 'team-booking'));
        $this->address->setHook('address');
        $this->address->setLabel(__('Address', 'team-booking'));
        $this->phone->setHook('phone');
        $this->phone->setLabel(__('Phone number', 'team-booking'));
        $this->url->setHook('url');
        $this->url->setLabel(__('Website', 'team-booking'));
        // Defaults
        $this->first_name->setOn();
        $this->first_name->setRequiredOn();
        $this->second_name->setOn();
        $this->second_name->setRequiredOn();
        $this->email->setOn();
        $this->email->setRequiredOn();
    }

    //------------------------------------------------------------

    public function getFirstName()
    {
        return $this->first_name;
    }

    public function getSecondName()
    {
        return $this->second_name;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getAddress()
    {
        return $this->address;
    }

    public function getPhone()
    {
        return $this->phone;
    }

    public function getUrl()
    {
        return $this->url;
    }

    //------------------------------------------------------------

    /**
     * @param bool $active_only
     *
     * @return TeamBookingFormTextField[]
     */
    public function getAllFields($active_only = FALSE)
    {
        /** @var $fields TeamBookingFormTextField[] */
        $fields = array_merge($this->getBuiltInFields(), $this->getCustomFields());
        if ($active_only) {
            foreach ($fields as $key => $field) {
                if ($field instanceof TeamBookingFormTextField) {
                    if (!$field->getIsActive()) {
                        unset($fields[ $key ]);
                    }
                } else {
                    unset($fields[ $key ]);
                }
            }
        }

        return $fields;
    }

    /**
     * @return TeamBookingFormTextField[]
     */
    public function getCustomFields()
    {
        return array_filter($this->custom_fields);
    }

    public function addCustomField(TeamBookingFormTextField $field)
    {
        $this->custom_fields[] = $field;
    }

    public function updateCustomField($old_hook, TeamBookingFormTextField $new_field)
    {
        foreach ($this->custom_fields as &$field) {
            if ($field instanceof TeamBookingFormTextField && $field->getHook() == $old_hook) {
                $field = $new_field;
            }
        }
    }

    public function dropCustomField($hook)
    {
        foreach ($this->custom_fields as $key => $field) {
            if ($field instanceof TeamBookingFormTextField && $field->getHook() == $hook) {
                unset($this->custom_fields[ $key ]);
                $this->custom_fields = array_values($this->custom_fields);
                break;
            }
        }
    }

    public function saveCustomFields(array $custom_fields)
    {
        $this->custom_fields = $custom_fields;
    }

    //------------------------------------------------------------

    public function getLabelFromHook($hook)
    {
        // Loop through built-in fields
        foreach ($this->getBuiltInFields() as $field) {
            if ($field->getHook() == $hook) {
                return $field->getLabel();
            }
        }
        // Loop through custom fields
        if (!empty($this->custom_fields)) {
            foreach ($this->custom_fields as $field) {
                if ($field instanceof TeamBookingFormTextField && $field->getHook() == $hook) {
                    return $field->getLabel();
                }
            }
        }

        return FALSE;
    }

    //------------------------------------------------------------

    /**
     * @return TeamBookingFormTextField[]
     */
    public function getBuiltInFields()
    {
        $return = array();
        $return['first_name'] = $this->first_name;
        $return['second_name'] = $this->second_name;
        $return['email'] = $this->email;
        $return['address'] = $this->address;
        $return['phone'] = $this->phone;
        $return['url'] = $this->url;

        return $return;
    }

    //------------------------------------------------------------

    public function isHookDuplicate($hook)
    {
        $hooks = $this->getHookList(TRUE);

        return in_array($hook, $hooks);
    }

    //------------------------------------------------------------

    /**
     * List of active hooks
     *
     * @param bool|FALSE $files_too
     *
     * @return array
     */
    public function getHookList($files_too = FALSE)
    {
        $return = array();
        // Loop through built-in fields
        foreach ($this->getBuiltInFields() as $field) {
            if ($field->getIsActive()) {
                $return[] = $field->getHook();
            }
        }
        // Loop through custom fields
        if (!empty($this->custom_fields)) {
            foreach ($this->custom_fields as $field) {
                if ($field instanceof TeamBookingFormTextField && $field->getIsActive()) {
                    if ($files_too == TRUE || get_class($field) !== 'TeamBookingFormFileUpload') {
                        $return[] = $field->getHook();
                    }
                }
            }
        }

        return $return;
    }

    //------------------------------------------------------------

    public function getPriceIncrementFromOptionValue($hook, $value)
    {
        foreach ($this->getCustomFields() as $field) {
            if ($field->getHook() != $hook) {
                continue;
            }
            if (get_class($field) === 'TeamBookingFormSelect'
                || get_class($field) === 'TeamBookingFormRadio'
            ) {
                /** @var $field TeamBookingFormSelect */
                if ($field->getValue() instanceof TeamBooking_Components_Form_Option && $field->getValue()->getText() == $value) {
                    return $field->getValue()->getPriceIncrement();
                }
                $options = $field->getOptions();
                foreach ($options as $option) {
                    if ($option->getText() == $value) {
                        return $option->getPriceIncrement();
                    }
                }
            }
        }

        return FALSE;
    }
}
