<?php

namespace TeamBooking;
defined('ABSPATH') or die('No script kiddies please!');

class Update
{
    /**
     * The plugin's Update method
     * Handles changes eventually needed in a not clean-install
     *
     * NOTES:
     *
     * - version < 1.2.5: it repairs the hooks for builtin fields
     *                    due to visibility properties change.
     *                    It resets the show/required settings.
     * - version < 1.3.0: creates the database tables for all
     *                    the reservations records, and transfer
     *                    the logs there.
     * - version < 1.4.0: updates the PayPal gateway by the new
     *                    interface, adds Stripe gateway too.
     * - version < 1.4.3: creates the database table for advanced
     *                    pricing features and sets the cleaning
     *                    cronjob.
     * - version < 1.4.5: sets the e-mail reminder cronjob.
     * - version < 2.0.0: creates the events table
     *
     */
    private static function doUpdate()
    {
        /* @var $settings \TeamBookingSettings */
        $settings = get_option('team_booking');
        $stored_version = $settings->getVersion();

        if (version_compare($stored_version, '1.2.5', '<')) {
            self::from1_2_5($settings);
        }
        if (version_compare($stored_version, '1.3.0', '<')) {
            self::from1_3_0($settings);
        }
        if (version_compare($stored_version, '1.4.0', '<')) {
            self::from1_4_0($settings);
        }
        if (version_compare($stored_version, '1.4.3', '<')) {
            self::from1_4_3();
        }
        if (version_compare($stored_version, '1.4.5', '<')) {
            self::from1_4_5();
        }
        if (version_compare($stored_version, '2.0.0', '<')) {
            self::from2_0_0($settings);
        }
        if (version_compare($stored_version, '2.1.0', '<')) {
            self::from2_1_0($settings);
        }
        if (version_compare($stored_version, '2.1.4', '<')) {
            self::from2_1_4();
        }
        $settings->setVersion(TEAMBOOKING_VERSION);
        $settings->save();

        $message = esc_html__('TeamBooking is now updated!', 'team-booking');
        $message_add = ' ' . esc_html__('Since there are major changes under the hood, all the Google Calendars must be re-synced!', 'team-booking');
        $button = '<a class="button button-primary" href="' . admin_url('admin.php?page=team-booking&whatsnew') . '">' . esc_html__("What's new?", 'team-booking') . '</a>';
        Admin\Framework\Notice::getUpdate($message, $button)->render();
    }

    private static function from1_2_5(\TeamBookingSettings $settings)
    {
        $services = $settings->getServices();
        $old_hooks_and_labels = array(
            'first_name'  => __('First name', 'team-booking'),
            'second_name' => __('Last name', 'team-booking'),
            'email'       => __('Email', 'team-booking'),
            'address'     => __('Address', 'team-booking'),
            'phone'       => __('Phone number', 'team-booking'),
            'url'         => __('Website', 'team-booking'),
        );
        $old_hooks = array_keys($old_hooks_and_labels);
        foreach ($services as $service) {
            /* @var $service \TeamBookingType */
            $fields = $service->getFormFields();
            $i = 0;
            foreach ($fields->getBuiltInFields() as $field) {
                $field->setHook($old_hooks[ $i ]);
                $field->setLabel($old_hooks_and_labels[ $old_hooks[ $i ] ]);
                $i++;
            }
            $service->setFormFields($fields);
            $settings->updateService($service, $service->getId());
        }
    }

    private static function from1_3_0(\TeamBookingSettings $settings)
    {
        global $wpdb;
        $table_name = \TeamBooking_Install::createReservationsTable();
        $logs = $settings->getLogs();
        foreach ($logs as $key => $log) {
            if ($log instanceof \TeamBooking_ReservationData) {
                /* @var $log \TeamBooking_ReservationData */
                $log->setCreationInstant($key);
                $created = current_time('Y-m-d H:i:s');
                $data_object = serialize($log);
                $wpdb->insert($table_name, array(
                    'created'     => $created,
                    'service_id'  => $log->getServiceId(),
                    'coworker_id' => $log->getCoworker(),
                    'data_object' => $data_object,
                ));
                $settings->dropLog($key);
            }
        }
    }

    private static function from1_4_0(\TeamBookingSettings $settings)
    {
        $payment_gateways = $settings->getPaymentGatewaySettingObjects();
        if (empty($payment_gateways)) {
            $settings->addPaymentGatewaySettingObject(new \TeamBooking_PaymentGateways_Stripe_Settings());
            $settings->addPaymentGatewaySettingObject(new \TeamBooking_PaymentGateways_PayPal_Settings());
        }
        $settings->clean();
    }

    private static function from1_4_3()
    {
        \TeamBooking_Install::createPromotionsTable();
        if (!wp_get_schedule('tb-db-cleaning-routine')) {
            wp_schedule_event(time(), 'daily', 'tb-db-cleaning-routine');
        }
    }

    private static function from1_4_5()
    {
        if (!wp_get_schedule('tb_email_reminder_handler')) {
            wp_schedule_event(time(), 'hourly', 'tb_email_reminder_handler');
        }
    }

    private static function from2_0_0(\TeamBookingSettings $settings)
    {
        global $wpdb;
        \TeamBooking_Install::createEventsTable();
        foreach ($settings->getCoworkersData() as $coworker_data) {
            $new_array = array();
            if (NULL === $coworker_data->getApiToken()) {
                $coworker_data->refreshApiToken();
            }
            foreach ($coworker_data->getCalendars() as $calendar) {
                if (is_array($calendar)) {
                    $new_array[ $calendar['calendar_id'] ] = array('calendar_id' => $calendar['calendar_id'], 'sync_token' => NULL);
                } else {
                    $new_array[ $calendar ] = array('calendar_id' => $calendar, 'sync_token' => NULL);
                }
            }
            if (!empty($new_array)) {
                $coworker_data->addCalendars($new_array);
                $settings->updateCoworkerData($coworker_data);
            }
        }
        // Add new columns
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $columns = $wpdb->get_col("DESC {$table_name}", 0);
        if (!in_array('calendar_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD calendar_id text");
        if (!in_array('event_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD event_id text");
        if (!in_array('status', $columns)) $wpdb->query("ALTER TABLE $table_name ADD status text");
        foreach (Database\Reservations::getAll() as $id => $data) {
            if (!$data->getToken()) {
                $data->setToken(Toolkit\generateToken());
            }
            Database\Reservations::update($data);
        }
        $table_name = $wpdb->prefix . 'teambooking_reservations_pending';
        if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") == $table_name) {
            $results = $wpdb->get_results("SELECT * FROM $table_name");
            $return = array();
            foreach ($results as $result) {
                $return[ $result->id ] = unserialize($result->data_object);
            }
            foreach ($return as $data) {
                /** @var $data \TeamBooking_ReservationData */
                $data->setStatusPending();
                Database\Reservations::insert($data);
            }
            $wpdb->query("DROP TABLE IF EXISTS $table_name");
        }
    }

    private static function from2_1_0(\TeamBookingSettings $settings)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_events';
        $wpdb->query("TRUNCATE TABLE $table_name");
        $columns = $wpdb->get_col("DESC {$table_name}", 0);
        if (!in_array('organizer', $columns)) $wpdb->query("ALTER TABLE $table_name ADD organizer text");
        if (!in_array('guests', $columns)) $wpdb->query("ALTER TABLE $table_name ADD guests text");
        if (!in_array('allday', $columns)) $wpdb->query("ALTER TABLE $table_name ADD allday tinyint");
        if (in_array('encoded_event', $columns)) $wpdb->query("ALTER TABLE $table_name DROP COLUMN encoded_event");
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $columns = $wpdb->get_col("DESC {$table_name}", 0);
        if (!in_array('calendar_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD calendar_id text");
        if (!in_array('event_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD event_id text");
        if (!in_array('status', $columns)) $wpdb->query("ALTER TABLE $table_name ADD status text");
        if (!in_array('token', $columns)) $wpdb->query("ALTER TABLE $table_name ADD token text");
        foreach (Database\Reservations::getAll() as $id => $data) {
            if (!$data->getToken()) {
                $data->setToken(Toolkit\generateToken());
            }
            Database\Reservations::update($data);
        }
        foreach ($settings->getCoworkersData() as $c_data) {
            $c_data->cleanSyncTokens();
        }
    }

    private static function from2_1_4()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $columns = $wpdb->get_col("DESC {$table_name}", 0);
        if (!in_array('start', $columns)) $wpdb->query("ALTER TABLE $table_name ADD start int");
        if (!in_array('end', $columns)) $wpdb->query("ALTER TABLE $table_name ADD end int");
        foreach (Database\Reservations::getAll() as $id => $data) {
            if ($data->getSlotStart() !== NULL) {
                $data->setStart(strtotime($data->getSlotStart()));
                $data->setEnd(strtotime($data->getSlotEnd()));
            }
            Database\Reservations::update($data);
        }
    }

    public static function update()
    {
        global $wpdb;
        if (function_exists('is_multisite') && is_multisite()) {
            if (!function_exists('is_plugin_active_for_network')) {
                require_once ABSPATH . '/wp-admin/includes/plugin.php';
            }
            if (is_plugin_active_for_network(plugin_basename(TEAMBOOKING_FILE_PATH))) {
                foreach ($wpdb->get_col("SELECT blog_id FROM $wpdb->blogs") as $blog_id) {
                    switch_to_blog($blog_id);
                    self::doUpdate();
                    restore_current_blog();
                }
            } else {
                self::doUpdate();
            }
        } else {
            self::doUpdate();
        }
    }

    public static function repairDatabase()
    {
        global $wpdb;
        if (function_exists('is_multisite') && is_multisite()) {
            if (!function_exists('is_plugin_active_for_network')) {
                require_once ABSPATH . '/wp-admin/includes/plugin.php';
            }
            if (is_plugin_active_for_network(plugin_basename(TEAMBOOKING_FILE_PATH))) {
                foreach ($wpdb->get_col("SELECT blog_id FROM $wpdb->blogs") as $blog_id) {
                    switch_to_blog($blog_id);
                    self::doRepair();
                    restore_current_blog();
                }
            } else {
                self::doRepair();
            }
        } else {
            self::doRepair();
        }
    }

    private static function doRepair()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_events';
        $wpdb->query("TRUNCATE TABLE $table_name");
        $columns = $wpdb->get_col("DESC {$table_name}", 0);
        if (!in_array('description', $columns)) $wpdb->query("ALTER TABLE $table_name ADD description text");
        if (!in_array('hangout_link', $columns)) $wpdb->query("ALTER TABLE $table_name ADD hangout_link text");
        if (!in_array('html_link', $columns)) $wpdb->query("ALTER TABLE $table_name ADD html_link text");
        if (!in_array('location', $columns)) $wpdb->query("ALTER TABLE $table_name ADD location text");
        if (!in_array('recurrence', $columns)) $wpdb->query("ALTER TABLE $table_name ADD recurrence text");
        if (!in_array('recurring_event_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD recurring_event_id text");
        if (!in_array('summary', $columns)) $wpdb->query("ALTER TABLE $table_name ADD summary text");
        if (!in_array('color_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD color_id int");
        if (!in_array('event_start', $columns)) $wpdb->query("ALTER TABLE $table_name ADD event_start int");
        if (!in_array('event_end', $columns)) $wpdb->query("ALTER TABLE $table_name ADD event_end int");
        if (!in_array('organizer', $columns)) $wpdb->query("ALTER TABLE $table_name ADD organizer text");
        if (!in_array('guests', $columns)) $wpdb->query("ALTER TABLE $table_name ADD guests text");
        if (!in_array('allday', $columns)) $wpdb->query("ALTER TABLE $table_name ADD allday tinyint");
        if (in_array('encoded_event', $columns)) $wpdb->query("ALTER TABLE $table_name DROP COLUMN encoded_event");
        foreach (\TeamBooking\Functions\getSettings()->getCoworkersData() as $c_data) {
            $c_data->cleanSyncTokens();
        }
        \TeamBooking\Functions\getSettings()->save();
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $columns = $wpdb->get_col("DESC {$table_name}", 0);
        if (!in_array('calendar_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD calendar_id text");
        if (!in_array('event_id', $columns)) $wpdb->query("ALTER TABLE $table_name ADD event_id text");
        if (!in_array('status', $columns)) $wpdb->query("ALTER TABLE $table_name ADD status text");
        if (!in_array('token', $columns)) $wpdb->query("ALTER TABLE $table_name ADD token text");
        if (!in_array('start', $columns)) $wpdb->query("ALTER TABLE $table_name ADD start int");
        if (!in_array('end', $columns)) $wpdb->query("ALTER TABLE $table_name ADD end int");
        foreach (Database\Reservations::getAll() as $id => $data) {
            if (!$data->getToken()) {
                $data->setToken(Toolkit\generateToken());
            }
            if ($data->getSlotStart() !== NULL) {
                $data->setStart(strtotime($data->getSlotStart()));
                $data->setEnd(strtotime($data->getSlotEnd()));
            }
            Database\Reservations::update($data);
        }
    }
}